@@ -95,6 +95,9 @@
 #include "screen.h"
 #include "icons.h"
 #include "iconmgr.h"
+#ifdef X11R6
+#  include "session.h"
+#endif
 
 #define gray_width 2
 #define gray_height 2
@@ -105,17 +108,22 @@ static unsigned char black_bits[] = {
 
 int AddingX;
 int AddingY;
-int AddingW;
-int AddingH;
+unsigned int AddingW;
+unsigned int AddingH;
 
 static int PlaceX = 50;
 static int PlaceY = 50;
-static void CreateWindowTitlebarButtons();
+static void CreateWindowTitlebarButtons(TwmWindow *tmp_win);
 
-static void		splitWindowRegionEntry ();
-static WindowEntry	*findWindowEntry ();
-static WindowEntry	*prevWindowEntry ();
-static void		mergeWindowEntries ();
+static void		splitWindowRegionEntry (WindowEntry	*we,
+						int grav1, int grav2,
+						int w, int h);
+static WindowEntry	*findWindowEntry (WorkSpace    *wl,
+					  TwmWindow    *tmp_win,
+					  WindowRegion **wrp);
+static WindowEntry	*prevWindowEntry (WindowEntry	*we,
+					  WindowRegion	*wr);
+static void		mergeWindowEntries (WindowEntry	*old, WindowEntry *we);
 
 char NoName[] = "Untitled"; /* name if no name is specified */
 int  resizeWhenAdd;
@@ -137,9 +145,8 @@ extern int captive;
  ************************************************************************
  */
 
-GetGravityOffsets (tmp, xp, yp)
-    TwmWindow *tmp;			/* window from which to get gravity */
-    int *xp, *yp;			/* return values */
+void GetGravityOffsets (TwmWindow *tmp,	/* window from which to get gravity */
+			int *xp, int *yp)	/* return values */
 {
     static struct _gravity_offset {
 	int x, y;
@@ -191,12 +198,9 @@ GetGravityOffsets (tmp, xp, yp)
  ***********************************************************************
  */
 
-TwmWindow *
-AddWindow(w, iconm, iconp)
-Window w;
-int iconm;
-IconMgr *iconp;
+TwmWindow *AddWindow(Window w, int iconm, IconMgr *iconp)
 {
+    virtualScreen *vs;
     TwmWindow *tmp_win;			/* new twm window structure */
     int stat;
     XEvent event;
@@ -223,7 +227,7 @@ IconMgr *iconp;
     int saved_occupation; /* <== [ Matthew McNeill Feb 1997 ] == */
 #endif
     Bool        random_placed = False;
-    int		found;
+    int		found = 0;
 #ifndef VMS
     fd_set	mask;
     int		fd;
@@ -234,7 +238,9 @@ IconMgr *iconp;
     XRectangle logical_rect;
 #endif
     WindowBox *winbox;
-    int iswinbox;
+    int iswinbox = 0;
+    int iswman = 0;
+    Window vroot;
 
 #ifdef DEBUG
     fprintf(stderr, "AddWindow: w = 0x%x\n", w);
@@ -253,21 +259,24 @@ IconMgr *iconp;
     switch (iconm) {
 	case  0 : iswinbox = 0; break;
 	case  1 : iswinbox = 0; break;
-	case  2 : iswinbox = 1; iconm = 0; break;
-	default : iswinbox = 0; iconm = 1; break;
+	case  2 : iswinbox = 1; iconm  = 0; break;
+	case  3 : iswman   = 1; iconm  = 0; break;
+	default : iswinbox = 0; iswman = 0; iconm = 1; break;
     }
     tmp_win->w = w;
     tmp_win->zoomed = ZOOM_NONE;
     tmp_win->iconmgr = iconm;
     tmp_win->iconmgrp = iconp;
+    tmp_win->wspmgr = iswman;
     tmp_win->iswinbox = iswinbox;
+    tmp_win->vs = NULL;
     tmp_win->cmaps.number_cwins = 0;
     tmp_win->savegeometry.width = -1;
 
     XSelectInput(dpy, tmp_win->w, PropertyChangeMask);
     XGetWindowAttributes(dpy, tmp_win->w, &tmp_win->attr);
 #ifndef NO_LOCALE
-    tmp_win->name = GetWMPropertyString(tmp_win->w, XA_WM_NAME);
+    tmp_win->name = (char*) GetWMPropertyString(tmp_win->w, XA_WM_NAME);
 #else /* NO_LOCALE */
     XFetchName(dpy, tmp_win->w, &tmp_win->name);
 #endif /* NO_LOCALE */
@@ -365,8 +374,17 @@ IconMgr *iconp;
     	tmp_win->class.res_class = NoName;
 
     tmp_win->full_name = tmp_win->name;
+#ifdef CLAUDE
+    if (strstr (tmp_win->name, " - Mozilla")) {
+      char *moz = strstr (tmp_win->name, " - Mozilla");
+      *moz = '\0';
+    }
+#endif
     namelen = strlen (tmp_win->name);
 
+    if (LookInList(Scr->IgnoreTransientL, tmp_win->full_name, &tmp_win->class))
+      tmp_win->transient = 0;
+
     tmp_win->highlight = Scr->Highlight && 
 	(!LookInList(Scr->NoHighlight, tmp_win->full_name, 
 	    &tmp_win->class));
@@ -379,7 +397,7 @@ IconMgr *iconp;
 	tmp_win->full_name, &tmp_win->class)) ? ONTOP_MAX : ONTOP_DEFAULT;
 
     tmp_win->titlehighlight = Scr->TitleHighlight && 
-	(!(short)(int) LookInList(Scr->NoTitleHighlight, tmp_win->full_name, 
+	(!LookInList(Scr->NoTitleHighlight, tmp_win->full_name, 
 	    &tmp_win->class));
 
     tmp_win->auto_raise = Scr->AutoRaiseDefault ||
@@ -396,12 +414,11 @@ IconMgr *iconp;
     if (Scr->IconifyByUnmapping)
     {
 	tmp_win->iconify_by_unmapping = iconm ? FALSE :
-	    !(short)(int) LookInList(Scr->DontIconify, tmp_win->full_name,
+	    !LookInList(Scr->DontIconify, tmp_win->full_name,
 		&tmp_win->class);
     }
-    tmp_win->iconify_by_unmapping |= 
-	(short)(int) LookInList(Scr->IconifyByUn, tmp_win->full_name,
-	    &tmp_win->class);
+    tmp_win->iconify_by_unmapping = tmp_win->iconify_by_unmapping ||
+	LookInList(Scr->IconifyByUn, tmp_win->full_name, &tmp_win->class);
 
     if (LookInList (Scr->UnmapByMovingFarAway, tmp_win->full_name, &tmp_win->class))
 	tmp_win->UnmapByMovingFarAway = True;
@@ -423,6 +440,12 @@ IconMgr *iconp;
     else
 	tmp_win->StartSqueezed = False;
 
+    if (Scr->AlwaysSqueezeToGravity
+	|| LookInList (Scr->AlwaysSqueezeToGravityL, tmp_win->full_name, &tmp_win->class))
+	tmp_win->AlwaysSqueezeToGravity = True;
+    else
+	tmp_win->AlwaysSqueezeToGravity = False;
+
     if (tmp_win->transient || tmp_win->group) {
 	TwmWindow *t;
 	for (t = Scr->TwmRoot.next; t != NULL; t = t->next) {
@@ -431,7 +454,7 @@ IconMgr *iconp;
 	}
 	if (t) tmp_win->UnmapByMovingFarAway = t->UnmapByMovingFarAway;
     }
-    if ((Scr->WindowRingAll &&
+    if ((Scr->WindowRingAll && !iswman &&
 	!LookInList(Scr->WindowRingExcludeL, tmp_win->full_name, &tmp_win->class)) ||
 	LookInList(Scr->WindowRingL, tmp_win->full_name, &tmp_win->class)) {
 	if (Scr->Ring) {
@@ -568,12 +591,9 @@ IconMgr *iconp;
 #ifdef X11R6
     if (restoredFromPrevSession) {
       SetupOccupation (tmp_win, saved_occupation);
-    } else {
-      SetupOccupation (tmp_win, 0);
-    }
-#else
-      SetupOccupation (tmp_win, 0);
+    } else
 #endif      
+      SetupOccupation (tmp_win, 0);
     /*=================================================================*/
 
     tmp_win->frame_width  = tmp_win->attr.width  + 2 * tmp_win->frame_bw3D;
@@ -586,17 +606,26 @@ IconMgr *iconp;
     }
     if (LookInList (Scr->WindowGeometries, tmp_win->full_name, &tmp_win->class)) {
         char *geom;
-	int mask;
+	int mask_;
 	geom = LookInList (Scr->WindowGeometries, tmp_win->full_name, &tmp_win->class);
-	mask= XParseGeometry (geom, &tmp_win->attr.x, &tmp_win->attr.y,
-			      (unsigned int*) &tmp_win->attr.width,
-			      (unsigned int*) &tmp_win->attr.height);
+	mask_ = XParseGeometry (geom, &tmp_win->attr.x, &tmp_win->attr.y,
+				(unsigned int*) &tmp_win->attr.width,
+				(unsigned int*) &tmp_win->attr.height);
 
-	if (mask & XNegative) tmp_win->attr.x += Scr->MyDisplayWidth  - tmp_win->attr.width;
-	if (mask & YNegative) tmp_win->attr.y += Scr->MyDisplayHeight - tmp_win->attr.height;
-        random_placed = True;
+	if (mask_ & XNegative) tmp_win->attr.x += Scr->rootw - tmp_win->attr.width;
+	if (mask_ & YNegative) tmp_win->attr.y += Scr->rooth - tmp_win->attr.height;
 	ask_user = False;
     }
+
+    if (XFindContext (dpy, w, VirtScreenContext, (XPointer *)&vs) == XCSUCCESS)
+      vroot = vs->window;
+    else
+    if (tmp_win->vs)
+      vroot = tmp_win->vs->window;
+    else
+      vroot = Scr->Root;
+    if (winbox) vroot = winbox->window;
+
     /*
      * do any prompting for position
      */
@@ -609,16 +638,128 @@ IconMgr *iconp;
       if ((Scr->RandomPlacement == RP_ALL) ||
           ((Scr->RandomPlacement == RP_UNMAPPED) &&
 	   ((tmp_win->wmhints && (tmp_win->wmhints->initial_state == IconicState)) ||
-	    (! OCCUPY (tmp_win, Scr->workSpaceMgr.activeWSPC))))) { /* just stick it somewhere */
-	if ((PlaceX + tmp_win->attr.width) > Scr->MyDisplayWidth)
-	    PlaceX = 50;
-	if ((PlaceY + tmp_win->attr.height) > Scr->MyDisplayHeight)
-	    PlaceY = 50;
+	    (! visible (tmp_win))))) { /* just stick it somewhere */
 
+#ifdef DEBUG
+	fprintf(stderr,
+		"DEBUG[RandomPlacement]: win: %dx%d+%d+%d, screen: %dx%d, title height: %d, random: +%d+%d\n",
+		tmp_win->attr.width, tmp_win->attr.height,
+		tmp_win->attr.x, tmp_win->attr.y,
+		Scr->rootw, Scr->rooth,
+		tmp_win->title_height,
+		PlaceX, PlaceY);
+#endif
+
+	/* If the right edge of the window would fall outside of the
+	   screen, start over by placing the left edge of the window
+	   50 pixels inside the left edge of the screen.*/
+	if ((PlaceX + tmp_win->attr.width)  > Scr->rootw)
+	  PlaceX = 50;
+
+	/* If the bottom edge of the window would fall outside of the
+	   screen, start over by placing the top edge of the window
+	   50 pixels inside the top edge of the screen.  Because we
+	   add the title height further down, we need to count with
+	   it here as well.  */
+	if ((PlaceY + tmp_win->attr.height
+		 + tmp_win->title_height) > Scr->rooth)
+	  PlaceY = 50;
+
+	/* Assign the current random placement to the new window, as
+	   a preliminary measure.  Add the title height so things will
+	   look right.  */
 	tmp_win->attr.x = PlaceX;
-	tmp_win->attr.y = PlaceY;
-	PlaceX += 30;
-	PlaceY += 30;
+	tmp_win->attr.y = PlaceY + tmp_win->title_height;
+
+	/* If the window is not supposed to move off the screen, check
+	   that it's still within the screen, and if not, attempt to
+	   correct the situation. */
+	if (Scr->DontMoveOff) {
+	    int available;
+
+#ifdef DEBUG
+	    fprintf(stderr,
+		    "DEBUG[DontMoveOff]: win: %dx%d+%d+%d, screen: %dx%d, bw2: %d, bw3D: %d\n",
+		    tmp_win->attr.width, tmp_win->attr.height,
+		    tmp_win->attr.x, tmp_win->attr.y,
+		    Scr->rootw, Scr->rooth,
+		    bw2, tmp_win->frame_bw3D);
+#endif
+
+	    /* If the right edge of the window is outside the right edge
+	       of the screen, we need to move the window left.  Note that
+	       this can only happen with windows that are less than 50
+	       pixels less wide than the screen. */
+	    if ((tmp_win->attr.x + tmp_win->attr.width)  > Scr->rootw) {
+	      available = Scr->rootw - tmp_win->attr.width
+		- 2 * (bw2 + tmp_win->frame_bw3D);
+
+#ifdef DEBUG
+	      fprintf(stderr, "DEBUG[DontMoveOff]: availableX: %d\n",
+		      available);
+#endif
+
+	      /* If the window is wider than the screen or exactly the width
+		 of the screen, the availability is exactly 0.  The result
+		 will be to have the window placed as much to the left as
+		 possible. */
+	      if (available <= 0) available = 0;
+
+	      /* Place the window exactly between the left and right edge of
+		 the screen when possible.  If available was originally less
+		 than zero, it means the window's left edge will be against
+		 the screen's left edge, and the window's right edge will be
+		 outside the screen.  */
+	      tmp_win->attr.x = available / 2;
+	    }
+
+	    /* If the bottom edge of the window is outside the bottom edge
+	       of the screen, we need to move the window up.  Note that
+	       this can only happen with windows that are less than 50
+	       pixels less tall than the screen.  Don't forget to count
+	       with the title height and the frame widths.  */
+	    if ((tmp_win->attr.y + tmp_win->attr.height)  > Scr->rooth) {
+	      available = Scr->rooth - tmp_win->attr.height
+		- tmp_win->title_height - 2 * (bw2 + tmp_win->frame_bw3D);
+
+#ifdef DEBUG
+	      fprintf(stderr, "DEBUG[DontMoveOff]: availableY: %d\n",
+		      available);
+#endif
+
+	      /* If the window is taller than the screen or exactly the
+		 height of the screen, the availability is exactly 0.
+		 The result will be to have the window placed as much to
+		 the top as possible. */
+	      if (available <= 0) available = 0;
+
+	      /* Place the window exactly between the top and bottom edge of
+		 the screen when possible.  If available was originally less
+		 than zero, it means the window's top edge will be against
+		 the screen's top edge, and the window's bottom edge will be
+		 outside the screen.  Again, don't forget to add the title
+		 height.  */
+	      tmp_win->attr.y = available / 2 + tmp_win->title_height;
+	    }
+
+#ifdef DEBUG
+	    fprintf(stderr,
+		    "DEBUG[DontMoveOff]: win: %dx%d+%d+%d, screen: %dx%d\n",
+		    tmp_win->attr.width, tmp_win->attr.height,
+		    tmp_win->attr.x, tmp_win->attr.y,
+		    Scr->rootw, Scr->rooth);
+#endif
+	}
+
+	/* We know that if the window's left edge has moved compared to
+	   PlaceX, it will have moved to the left.  If it was moved less
+	   than 15 pixel either way, change the next "random position"
+	   30 pixels down and right. */
+	if (PlaceX - tmp_win->attr.x < 15
+	    || PlaceY - tmp_win->attr.y < 15) {
+	  PlaceX += 30;
+	  PlaceY += 30;
+	}
 
         random_placed = True;
       } else {				/* else prompt */
@@ -651,15 +792,17 @@ IconMgr *iconp;
 		if (firsttime) {
 		    if (JunkRoot != Scr->Root) {
 			register int scrnum;
-
 			for (scrnum = 0; scrnum < NumScreens; scrnum++) {
 			    if (JunkRoot == RootWindow (dpy, scrnum)) break;
 			}
-
 			if (scrnum != NumScreens) PreviousScreen = scrnum;
 		    }
+		    if (Scr->currentvs) {
+		      vroot = Scr->currentvs->window;
+		    }
 		    firsttime = False;
 		}
+		if (winbox) vroot = winbox->window;
 
 		/*
 		 * wait for buttons to come up; yuck
@@ -669,14 +812,12 @@ IconMgr *iconp;
 		/* 
 		 * this will cause a warp to the indicated root
 		 */
-		stat = XGrabPointer(dpy, winbox ? winbox->window : Scr->Root, False,
+		stat = XGrabPointer(dpy, vroot, False,
 		    ButtonPressMask | ButtonReleaseMask |
 		    PointerMotionMask | PointerMotionHintMask,
 		    GrabModeAsync, GrabModeAsync,
-		    winbox ? winbox->window : Scr->Root, UpperLeftCursor, CurrentTime);
-
-		if (stat == GrabSuccess)
-		    break;
+		    vroot, UpperLeftCursor, CurrentTime);
+		if (stat == GrabSuccess) break;
 	    }
 
 #ifdef I18N
@@ -713,20 +854,15 @@ IconMgr *iconp;
 			      SIZE_HINDENT,
 			      SIZE_VINDENT + Scr->SizeFont.font->ascent,
 			      tmp_win->name, namelen);
-#endif	    
+#endif
 
-	    if (winbox) {
-		XTranslateCoordinates (dpy, Scr->Root, winbox->window,
-			AddingX, AddingY, &AddingX, &AddingY, &JunkChild);
-		ConstrainedToWinBox (tmp_win, AddingX, AddingY, &AddingX, &AddingY);
-	    }
+	    if (winbox) ConstrainedToWinBox (tmp_win, AddingX, AddingY, &AddingX, &AddingY);
 
 	    AddingW = tmp_win->attr.width + bw2 + 2 * tmp_win->frame_bw3D;
 	    AddingH = tmp_win->attr.height + tmp_win->title_height +
 				bw2 + 2 * tmp_win->frame_bw3D;
-		MoveOutline(winbox ? winbox->window : Scr->Root,
-			AddingX, AddingY, AddingW, AddingH,
-			    tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
+	    MoveOutline(vroot,AddingX, AddingY, AddingW, AddingH,
+			tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
 
 #ifdef I18N
 	    XmbDrawImageString (dpy, Scr->SizeWindow, Scr->SizeFont.font_set,
@@ -769,17 +905,11 @@ IconMgr *iconp;
 			if (event.type == ButtonPress)
 			    break;
 		}
-		event.xbutton.x_root -= Scr->MyDisplayX;
-		event.xbutton.y_root -= Scr->MyDisplayY;
-		
+		if (Scr->Root != Scr->RealRoot) FixRootEvent (&event);
 		if (event.type == ButtonPress) {
 		  AddingX = event.xbutton.x_root;
 		  AddingY = event.xbutton.y_root;
 
-		  if (winbox) {
-		    XTranslateCoordinates (dpy, Scr->Root, winbox->window,
-		    AddingX, AddingY, &AddingX, &AddingY, &JunkChild);
-		  }
 		  TryToGrid (tmp_win, &AddingX, &AddingY);
 		  if (Scr->PackNewWindows) TryToPack (tmp_win, &AddingX, &AddingY);
 
@@ -795,21 +925,16 @@ IconMgr *iconp;
 		    continue;
 		}
 
-		XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkChild,
+		XQueryPointer(dpy, vroot, &JunkRoot, &JunkChild,
 		    &JunkX, &JunkY, &AddingX, &AddingY, &JunkMask);
 
-		if (winbox) {
-		    XTranslateCoordinates (dpy, Scr->Root, winbox->window,
-			AddingX, AddingY, &AddingX, &AddingY, &JunkChild);
-		}
 		TryToGrid (tmp_win, &AddingX, &AddingY);
 		if (Scr->PackNewWindows) TryToPack (tmp_win, &AddingX, &AddingY);
 		if (Scr->DontMoveOff)
 		{
 		    ConstrainByBorders (tmp_win, &AddingX, AddingW, &AddingY, AddingH);
 		}
-		MoveOutline(winbox ? winbox->window : Scr->Root,
-			AddingX, AddingY, AddingW, AddingH,
+		MoveOutline(vroot, AddingX, AddingY, AddingW, AddingH,
 			    tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
 
 		DisplayPosition (tmp_win, AddingX, AddingY);
@@ -853,14 +978,14 @@ IconMgr *iconp;
 #undef HALF_AVE_CURSOR_SIZE
 		    dx += (tmp_win->frame_bw + 1);
 		    dy += (bw2 + tmp_win->title_height + 1);
-		    if (AddingX + dx >= Scr->MyDisplayWidth - Scr->BorderRight)
-		      dx = Scr->MyDisplayWidth - Scr->BorderRight - AddingX - 1;
-		    if (AddingY + dy >= Scr->MyDisplayHeight - Scr->BorderBottom)
-		      dy = Scr->MyDisplayHeight - Scr->BorderBottom - AddingY - 1;
+		    if (AddingX + dx >= Scr->rootw - Scr->BorderRight)
+		      dx = Scr->rootw - Scr->BorderRight - AddingX - 1;
+		    if (AddingY + dy >= Scr->rooth - Scr->BorderBottom)
+		      dy = Scr->rooth - Scr->BorderBottom - AddingY - 1;
 		    if (dx > 0 && dy > 0)
 		      XWarpPointer (dpy, None, None, 0, 0, 0, 0, dx, dy);
 		} else {
-		    XWarpPointer (dpy, None, Scr->Root, 0, 0, 0, 0,
+		    XWarpPointer (dpy, None, vroot, 0, 0, 0, 0,
 				  AddingX + AddingW/2, AddingY + AddingH/2);
 		}
 		AddStartResize(tmp_win, AddingX, AddingY, AddingW, AddingH);
@@ -879,8 +1004,7 @@ IconMgr *iconp;
 			    if (event.type == ButtonRelease)
 				break;
 		    }
-		    event.xbutton.x_root -= Scr->MyDisplayX;
-		    event.xbutton.y_root -= Scr->MyDisplayY;
+		    if (Scr->Root != Scr->RealRoot) FixRootEvent (&event);
 
 		    if (event.type == ButtonRelease)
 		    {
@@ -897,12 +1021,8 @@ IconMgr *iconp;
 		     * using multiple GXxor lines so that we don't need to 
 		     * grab the server.
 		     */
-		    XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkChild,
-			&JunkX, &JunkY, &AddingX, &AddingY, &JunkMask);
-		    if (winbox) {
-			XTranslateCoordinates (dpy, Scr->Root, winbox->window,
-			    AddingX, AddingY, &AddingX, &AddingY, &JunkChild);
-		    }
+		    XQueryPointer(dpy, vroot, &JunkRoot, &JunkChild,
+				  &JunkX, &JunkY, &AddingX, &AddingY, &JunkMask);
 
 		    if (lastx != AddingX || lasty != AddingY)
 		    {
@@ -918,8 +1038,8 @@ IconMgr *iconp;
 	    } 
 	    else if (event.xbutton.button == Button3)
 	    {
-		int maxw = Scr->MyDisplayWidth  - Scr->BorderRight  - AddingX - bw2;
-		int maxh = Scr->MyDisplayHeight - Scr->BorderBottom - AddingY - bw2;
+		int maxw = Scr->rootw - Scr->BorderRight  - AddingX - bw2;
+		int maxh = Scr->rooth - Scr->BorderBottom - AddingY - bw2;
 
 		/*
 		 * Make window go to bottom of screen, and clip to right edge.
@@ -939,7 +1059,7 @@ IconMgr *iconp;
 		XMaskEvent(dpy, ButtonReleaseMask, &event);
 	    }
 	  }
-	    MoveOutline(winbox ? winbox->window : Scr->Root, 0, 0, 0, 0, 0, 0);
+	    MoveOutline(vroot, 0, 0, 0, 0, 0, 0);
 	    XUnmapWindow(dpy, Scr->SizeWindow);
 	    UninstallRootColormap();
 	    XUngrabPointer(dpy, CurrentTime);
@@ -956,11 +1076,6 @@ IconMgr *iconp;
     } else {				/* put it where asked, mod title bar */
 	/* if the gravity is towards the top, move it by the title height */
 	if (gravy < 0) tmp_win->attr.y -= gravy * tmp_win->title_height;
-	if (winbox) {
-	  XTranslateCoordinates (dpy, Scr->Root, winbox->window,
-                       tmp_win->attr.x,  tmp_win->attr.y,
-		      &tmp_win->attr.x, &tmp_win->attr.y, &JunkChild);
-	}
     }
 
 #ifdef DEBUG
@@ -979,18 +1094,8 @@ IconMgr *iconp;
 
     tmp_win->title_width = tmp_win->attr.width;
 
-#ifdef I18N
-    XmbTextExtents(Scr->TitleBarFont.font_set, tmp_win->name, namelen,
-		   &ink_rect, &logical_rect);
-    
-    tmp_win->name_width = logical_rect.width;
-#else    
-    tmp_win->name_width = XTextWidth(Scr->TitleBarFont.font, tmp_win->name,
-				     namelen);
-#endif    
-
 #ifndef NO_LOCALE
-    tmp_win->icon_name = GetWMPropertyString(tmp_win->w, XA_WM_ICON_NAME);
+    tmp_win->icon_name = (char*) GetWMPropertyString(tmp_win->w, XA_WM_ICON_NAME);
 #else /* NO_LOCALE */
     if (XGetWindowProperty (dpy, tmp_win->w, XA_WM_ICON_NAME, 0L, 200L, False,
 			    XA_STRING, &actual_type, &actual_format, &nitems,
@@ -998,19 +1103,21 @@ IconMgr *iconp;
 	tmp_win->icon_name = tmp_win->name;
 #endif /* NO_LOCALE */
 
-    if (tmp_win->icon_name == NULL)
-	tmp_win->icon_name = tmp_win->name;
+    if (tmp_win->icon_name == NULL) tmp_win->icon_name = tmp_win->name;
 #ifdef CLAUDE
-    else if ((strlen (tmp_win->icon_name) > 11) &&
-	    (strncmp (tmp_win->icon_name, "Netscape: ", 10) == 0)) {
-	char *tmp;
-
-	tmp = strdup (tmp_win->icon_name + 10);
-	XFree ((char*) tmp_win->icon_name);
-	tmp_win->icon_name = tmp;
+    if (strstr (tmp_win->icon_name, " - Mozilla")) {
+      char *moz = strstr (tmp_win->icon_name, " - Mozilla");
+      *moz = '\0';
     }
 #endif
 
+#ifdef I18N
+    XmbTextExtents (Scr->TitleBarFont.font_set, tmp_win->name, namelen, &ink_rect, &logical_rect);
+    tmp_win->name_width = logical_rect.width;
+#else    
+    tmp_win->name_width = XTextWidth (Scr->TitleBarFont.font, tmp_win->name, namelen);
+#endif    
+
     if (tmp_win->old_bw) XSetWindowBorderWidth (dpy, tmp_win->w, 0);
 
     tmp_win->squeezed = FALSE;
@@ -1109,8 +1216,14 @@ IconMgr *iconp;
 	valuemask |= CWWinGravity;
     }
 
-    tmp_win->frame = XCreateWindow (dpy, winbox ? winbox->window : Scr->Root, tmp_win->frame_x,
-				    tmp_win->frame_y, 
+    if ((tmp_win->frame_x > Scr->rootw) ||
+	(tmp_win->frame_y > Scr->rooth) ||
+	((int)(tmp_win->frame_x + tmp_win->frame_width)  < 0) ||
+	((int)(tmp_win->frame_y + tmp_win->frame_height) < 0)) {
+      tmp_win->frame_x = 0;
+      tmp_win->frame_y = 0;
+    }
+    tmp_win->frame = XCreateWindow (dpy, vroot, tmp_win->frame_x, tmp_win->frame_y, 
 				    (unsigned int) tmp_win->frame_width,
 				    (unsigned int) tmp_win->frame_height,
 				    (unsigned int) tmp_win->frame_bw,
@@ -1145,12 +1258,12 @@ IconMgr *iconp;
     if (tmp_win->highlight)
     {
 	if (Scr->use3Dtitles && (Scr->Monochrome != COLOR))
-	    tmp_win->gray = XCreatePixmapFromBitmapData(dpy, Scr->Root, 
+	    tmp_win->gray = XCreatePixmapFromBitmapData(dpy, vroot, 
 		(char*)black_bits, gray_width, gray_height, 
 		tmp_win->border_tile.fore, tmp_win->border_tile.back,
 		Scr->d_depth);
 	else
-	    tmp_win->gray = XCreatePixmapFromBitmapData(dpy, Scr->Root, 
+	    tmp_win->gray = XCreatePixmapFromBitmapData(dpy, vroot, 
 		(char*)gray_bits, gray_width, gray_height, 
 		tmp_win->border_tile.fore, tmp_win->border_tile.back,
 		Scr->d_depth);
@@ -1202,11 +1315,10 @@ IconMgr *iconp;
 	tmp_win->wShaped = boundingShaped;
     }
 
-    if (!tmp_win->iconmgr &&
-	(tmp_win->w != Scr->workSpaceMgr.workspaceWindow.w) &&
-	(tmp_win->w != Scr->workSpaceMgr.occupyWindow.w))
+    if (!tmp_win->iconmgr &&! iswman &&
+	(tmp_win->w != Scr->workSpaceMgr.occupyWindow->w))
 	XAddToSaveSet(dpy, tmp_win->w);
-	
+
     XReparentWindow(dpy, tmp_win->w, tmp_win->frame, tmp_win->frame_bw3D,
 		tmp_win->title_height + tmp_win->frame_bw3D);
     /*
@@ -1274,14 +1386,30 @@ IconMgr *iconp;
 	}
     }
 
+    if (tmp_win->group && tmp_win->group != tmp_win->w) {
+      /*
+       * GTK windows often have a spurious "group leader" window which is
+       * never reported to us and therefore does not really exist. Look for
+       * that here. It is in fact a duplicate of the WM_CLIENT_LEADER
+       * property.
+       */
+
+      TwmWindow *groupleader; /* the current twm window */
+      stat = XFindContext(dpy, tmp_win->group, TwmContext, (XPointer *)&groupleader);
+      if (stat == XCNOENT)
+	groupleader = NULL;
+      if (!groupleader) {
+	tmp_win->group = tmp_win->w;
+      }
+    } 
+
     XUngrabServer(dpy);
 
     /* if we were in the middle of a menu activated function, regrab
      * the pointer 
      */
-    if (RootFunction)
-	ReGrab();
-    WMapAddWindow (tmp_win);
+    if (RootFunction) ReGrab();
+    if (!iswman) WMapAddWindow (tmp_win);
     SetPropsIfCaptiveCtwm (tmp_win);
     savegeometry (tmp_win);
     return (tmp_win);
@@ -1304,9 +1432,7 @@ IconMgr *iconp;
  ***********************************************************************
  */
 
-int
-MappedNotOverride(w)
-    Window w;
+int MappedNotOverride(Window w)
 {
     XWindowAttributes wa;
 
@@ -1316,19 +1442,17 @@ MappedNotOverride(w)
 
 
 /***********************************************************************
- * 
+ *
  *  Procedure:
  *      AddDefaultBindings - attach default bindings so that naive users
  *      don't get messed up if they provide a minimal twmrc.
  */
-static void do_add_binding (button, context, modifier, func)
-    int button, context, modifier;
-    int func;
-{ 
+static void do_add_binding (int button, int context, int modifier, int func)
+{
     AddFuncButton (button, context, modifier, func, NULL, NULL);
 }
 
-AddDefaultBindings ()
+void AddDefaultBindings (void)
 {
 #define NoModifierMask 0
 
@@ -1364,9 +1488,7 @@ AddDefaultBindings ()
 		pointer_mode, GrabModeAsync, None,  \
 		Scr->FrameCursor);
 
-void
-GrabButtons(tmp_win)
-TwmWindow *tmp_win;
+void GrabButtons(TwmWindow *tmp_win)
 {
     FuncButton *tmp;
     int i;
@@ -1420,9 +1542,7 @@ TwmWindow *tmp_win;
 #define ungrabkey(funckey, modifier, window) \
 	XUngrabKey (dpy, funckey->keycode, funckey->mods | modifier, window);
 
-void
-GrabKeys(tmp_win)
-TwmWindow *tmp_win;
+void GrabKeys(TwmWindow *tmp_win)
 {
     FuncKey *tmp;
     IconMgr *p;
@@ -1436,10 +1556,11 @@ TwmWindow *tmp_win;
 	switch (tmp->cont)
 	{
 	case C_WINDOW:
+	/* case C_WORKSPACE: */
 	    if (tmp->mods & AltMask) break;
 	    grabkey (tmp, 0, tmp_win->w);
 	    if (Scr->IgnoreLockModifier && !(tmp->mods & LockMask))
-	        grabkey (tmp, LockMask, tmp_win->w);
+		grabkey (tmp, LockMask, tmp_win->w);
 	    for (i = 0 ; i < 8 ; i++) {
 		if ((Scr->IgnoreModifier & ModifierMask [i]) &&
 		    !(tmp->mods & ModifierMask [i]))
@@ -1528,7 +1649,7 @@ TwmWindow *tmp_win;
     }
 }
 
-void ComputeCommonTitleOffsets ()
+void ComputeCommonTitleOffsets (void)
 {
     int buttonwidth = (Scr->TBInfo.width + Scr->TBInfo.pad);
 
@@ -1544,8 +1665,7 @@ void ComputeCommonTitleOffsets ()
 				Scr->TitlePadding);
 }
 
-static void CreateHighlightWindows (tmp_win)
-    TwmWindow *tmp_win;
+static void CreateHighlightWindows (TwmWindow *tmp_win)
 {
     XSetWindowAttributes attributes;	/* attributes for create windows */
     GC gc;
@@ -1637,8 +1757,7 @@ static void CreateHighlightWindows (tmp_win)
 		       Scr->d_visual, valuemask, &attributes);
 }
 
-static void CreateLowlightWindows (tmp_win)
-    TwmWindow *tmp_win;
+static void CreateLowlightWindows (TwmWindow *tmp_win)
 {
     XSetWindowAttributes attributes;    /* attributes for create windows */
     unsigned long valuemask;
@@ -1702,9 +1821,8 @@ static void CreateLowlightWindows (tmp_win)
 }
 
 
-void ComputeWindowTitleOffsets (tmp_win, width, squeeze)
-    TwmWindow *tmp_win;
-    Bool squeeze;
+void ComputeWindowTitleOffsets (TwmWindow *tmp_win, unsigned int width,
+				Bool squeeze)
 {
     int titlew = width - Scr->TBInfo.titlex - Scr->TBInfo.rightoff;
 
@@ -1755,8 +1873,7 @@ void ComputeWindowTitleOffsets (tmp_win, width, squeeze)
  * to take the frame_bw into account since we want (0,0) of the title window
  * to line up with (0,0) of the frame window.
  */
-void ComputeTitleLocation (tmp)
-    register TwmWindow *tmp;
+void ComputeTitleLocation (register TwmWindow *tmp)
 {
     tmp->title_x = tmp->frame_bw3D - tmp->frame_bw;
     tmp->title_y = tmp->frame_bw3D - tmp->frame_bw;
@@ -1806,8 +1923,7 @@ void ComputeTitleLocation (tmp)
 }
 
 
-static void CreateWindowTitlebarButtons (tmp_win)
-    TwmWindow *tmp_win;
+static void CreateWindowTitlebarButtons (TwmWindow *tmp_win)
 {
     unsigned long valuemask;		/* mask for create windows */
     XSetWindowAttributes attributes;	/* attributes for create windows */
@@ -1896,8 +2012,7 @@ static void CreateWindowTitlebarButtons (tmp_win)
 }
 
 
-SetHighlightPixmap (filename)
-    char *filename;
+void SetHighlightPixmap (char *filename)
 {
 #ifdef VMS
     char *ftemp;
@@ -1909,8 +2024,7 @@ SetHighlightPixmap (filename)
 }
 
 
-FetchWmProtocols (tmp)
-    TwmWindow *tmp;
+void FetchWmProtocols (TwmWindow *tmp)
 {
     unsigned long flags = 0L;
     Atom *protocols = NULL;
@@ -1930,9 +2044,7 @@ FetchWmProtocols (tmp)
     tmp->protocols = flags;
 }
 
-TwmColormap *
-CreateTwmColormap(c)
-    Colormap c;
+TwmColormap *CreateTwmColormap(Colormap c)
 {
     TwmColormap *cmap;
     cmap = (TwmColormap *) malloc(sizeof(TwmColormap));
@@ -1949,11 +2061,9 @@ CreateTwmColormap(c)
     return (cmap);
 }
 
-ColormapWindow *
-CreateColormapWindow(w, creating_parent, property_window)
-    Window w;
-    Bool creating_parent;
-    Bool property_window;
+ColormapWindow *CreateColormapWindow(Window w,
+				     Bool creating_parent,
+				     Bool property_window)
 {
     ColormapWindow *cwin;
     TwmColormap *cmap;
@@ -2005,9 +2115,8 @@ CreateColormapWindow(w, creating_parent, property_window)
 
     return (cwin);
 }
-		
-FetchWmColormapWindows (tmp)
-    TwmWindow *tmp;
+
+int FetchWmColormapWindows (TwmWindow *tmp)
 {
     register int i, j;
     Window *cmap_windows = NULL;
@@ -2015,12 +2124,12 @@ FetchWmColormapWindows (tmp)
     int number_cmap_windows = 0;
     ColormapWindow **cwins = NULL;
     int previously_installed;
-    extern void free_cwins();
 
     number_cmap_windows = 0;
 
-    if (/* SUPPRESS 560 */previously_installed = 
-       (Scr->cmapInfo.cmaps == &tmp->cmaps && tmp->cmaps.number_cwins)) {
+    if (/* SUPPRESS 560 */
+	(previously_installed =
+	 (Scr->cmapInfo.cmaps == &tmp->cmaps && tmp->cmaps.number_cwins))) {
 	cwins = tmp->cmaps.cwins;
 	for (i = 0; i < tmp->cmaps.number_cwins; i++)
 	    cwins[i]->colormap->state = 0;
@@ -2136,8 +2245,7 @@ FetchWmColormapWindows (tmp)
 }
 
 
-void GetWindowSizeHints (tmp)
-    TwmWindow *tmp;
+void GetWindowSizeHints (TwmWindow *tmp)
 {
     long supplied = 0;
 
@@ -2155,16 +2263,15 @@ void GetWindowSizeHints (tmp)
 	int right =  tmp->attr.x + tmp->attr.width + 2 * tmp->old_bw;
 	int bottom = tmp->attr.y + tmp->attr.height + 2 * tmp->old_bw;
 	tmp->hints.win_gravity = 
-	  gravs[((Scr->MyDisplayHeight - bottom <
+	  gravs[((Scr->rooth - bottom <
 		tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 2) |
-		((Scr->MyDisplayWidth - right   <
+		((Scr->rootw - right   <
 		tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 1)];
 	tmp->hints.flags |= PWinGravity;
     }
 }
 
-AnimateButton (tbw)
-TBWindow *tbw;
+void AnimateButton (TBWindow *tbw)
 {
     Image	*image;
     XSetWindowAttributes attr;
@@ -2176,8 +2283,7 @@ TBWindow *tbw;
     tbw->image = image->next;
 }
 
-AnimateHighlight (t)
-TwmWindow *t;
+void AnimateHighlight (TwmWindow *t)
 {
     Image	*image;
     XSetWindowAttributes attr;
@@ -2195,9 +2301,7 @@ TwmWindow *t;
     t->HiliteImage = image->next;
 }
 
-name_list **AddWindowRegion (geom, grav1, grav2)
-char *geom;
-int  grav1, grav2;
+name_list **AddWindowRegion (char *geom, int grav1, int grav2)
 {
     WindowRegion *wr;
     int mask;
@@ -2216,15 +2320,15 @@ int  grav1, grav2;
     mask = XParseGeometry (geom, &wr->x, &wr->y, (unsigned int*) &wr->w,
 						 (unsigned int*) &wr->h);
 
-    if (mask & XNegative) wr->x += Scr->MyDisplayWidth  - wr->w;
-    if (mask & YNegative) wr->y += Scr->MyDisplayHeight - wr->h;
+    if (mask & XNegative) wr->x += Scr->rootw - wr->w;
+    if (mask & YNegative) wr->y += Scr->rooth - wr->h;
 
     return (&(wr->clientlist));
 }
 
-void CreateWindowRegions () {
+void CreateWindowRegions (void) {
     WindowRegion  *wr, *wr1 = NULL, *wr2 = NULL;
-    WorkSpaceList *wl;
+    WorkSpace *wl;
 
     for (wl = Scr->workSpaceMgr.workSpaceList; wl != NULL; wl = wl->next) {
 	wl->FirstWindowRegion = NULL;
@@ -2248,14 +2352,12 @@ void CreateWindowRegions () {
 }
 
 
-Bool PlaceWindowInRegion (tmp_win, final_x, final_y)
-TwmWindow *tmp_win;
-int       *final_x, *final_y;
+Bool PlaceWindowInRegion (TwmWindow *tmp_win, int *final_x, int *final_y)
 {
     WindowRegion  *wr;
     WindowEntry	  *we;
-    int		   w, h;
-    WorkSpaceList *wl;
+    int		  w, h;
+    WorkSpace     *wl;
 
     if (!Scr->FirstWindowRegion) return (False);
     for (wl = Scr->workSpaceMgr.workSpaceList; wl != NULL; wl = wl->next) {
@@ -2286,10 +2388,8 @@ int       *final_x, *final_y;
     return (True);
 }
 
-static void splitWindowRegionEntry (we, grav1, grav2, w, h)
-WindowEntry	*we;
-int		grav1, grav2;
-int		w, h;
+static void splitWindowRegionEntry (WindowEntry *we, int grav1, int grav2,
+				    int w, int h)
 {
     WindowEntry	*new;
     int		save;
@@ -2340,10 +2440,8 @@ int		w, h;
     }
 }
 
-static WindowEntry *findWindowEntry (wl, tmp_win, wrp)
-WorkSpaceList	*wl;
-TwmWindow	*tmp_win;
-WindowRegion	**wrp;
+static WindowEntry *findWindowEntry (WorkSpace *wl, TwmWindow *tmp_win,
+				     WindowRegion **wrp)
 {
     WindowRegion *wr;
     WindowEntry	 *we;
@@ -2359,9 +2457,7 @@ WindowRegion	**wrp;
     return (WindowEntry*) 0;
 }
 
-static WindowEntry *prevWindowEntry (we, wr)
-WindowEntry	*we;
-WindowRegion	*wr;
+static WindowEntry *prevWindowEntry (WindowEntry *we, WindowRegion *wr)
 {
     WindowEntry	*wp;
 
@@ -2370,8 +2466,7 @@ WindowRegion	*wr;
     return wp;
 }
 
-static void mergeWindowEntries (old, we)
-WindowEntry	*old, *we;
+static void mergeWindowEntries (WindowEntry *old, WindowEntry *we)
 {
     if (old->y == we->y) {
 	we->w = old->w + we->w;
@@ -2382,12 +2477,11 @@ WindowEntry	*old, *we;
     }
 }
 
-void RemoveWindowFromRegion (tmp_win)
-TwmWindow	*tmp_win;
+void RemoveWindowFromRegion (TwmWindow *tmp_win)
 {
-    WindowEntry   *we, *wp, *wn;
-    WindowRegion  *wr;
-    WorkSpaceList *wl;
+    WindowEntry  *we, *wp, *wn;
+    WindowRegion *wr;
+    WorkSpace    *wl;
 
     if (!Scr->FirstWindowRegion) return;
     we = (WindowEntry*) 0;
