@@ -73,6 +73,7 @@
 
 #include <stdio.h>
 #include <signal.h>
+#include <unistd.h>
 
 #ifdef __WAIT_FOR_CHILDS
 #  include <sys/wait.h>
@@ -84,6 +85,7 @@
 #include <fcntl.h>
 #endif
 #include "twm.h"
+#include "ctwm.h"
 #include "add_window.h"
 #include "gc.h"
 #include "parse.h"
@@ -95,6 +97,14 @@
 #include "screen.h"
 #include "icons.h"
 #include "iconmgr.h"
+#ifdef X11R6
+#  include "session.h"
+#endif
+#include "cursor.h"
+#include "windowbox.h"
+#ifdef SOUNDS
+#  include "sound.h"
+#endif
 #ifdef VMS
 #  include <stdlib.h>
 #  include <decw$include/Xproto.h>
@@ -102,9 +112,17 @@
 #  include <X11Xmu/Error.h>
 #  include "vms_cmd_services.h"
 
-#  ifndef NO_LOCALE
-#    include <locale.h>
-#  endif /* NO_LOCALE */
+#  ifdef X11R6
+#    include <X11SM/SMlib.h>
+#  endif /* X11R6 */
+
+#  ifdef I18N
+#    include <X11/Xlocale.h>
+#  else
+#    ifndef NO_LOCALE
+#      include <locale.h>
+#    endif /* NO_LOCALE */
+#  endif
 
 #  ifndef PIXMAP_DIRECTORY
 #    define PIXMAP_DIRECTORY "DECW$BITMAPS:"
@@ -118,32 +136,33 @@
 #    include <X11/SM/SMlib.h>
 #  endif /* X11R6 */
 
-#ifdef I18N
-#include <X11/Xlocale.h>
-#else
-#  ifndef NO_LOCALE
-#    include <locale.h>
-#  endif /* NO_LOCALE */
-#endif
+#  ifdef I18N
+#    include <X11/Xlocale.h>
+#  else
+#    ifndef NO_LOCALE
+#      include <locale.h>
+#    endif /* NO_LOCALE */
+#  endif
 
 #  ifndef PIXMAP_DIRECTORY
 #    define PIXMAP_DIRECTORY "/usr/lib/X11/twm"
 #  endif /* PIXMAP_DIRECTORY */
 #endif /* VMS */
 
-#  ifdef X11R6
+#ifdef X11R6
 XtAppContext appContext;	/* Xt application context */
 #endif /* X11R6 */
 
 Display *dpy;			/* which display are we talking to */
-#ifdef USEM4
 char *display_name = NULL;      /* JMO 2/13/90 for m4 */
+#ifdef USEM4
 int KeepTmpFile = False;        /* JMO 3/28/90 for m4 */
 char *keepM4_filename = NULL;	/* Keep M4 output here */
 int GoThroughM4 = True;
 #endif
 Window ResizeWindow;		/* the window we are resizing */
 
+int  cfgchk       = 0;
 int  captive      = FALSE;
 char *captivename = NULL;
 
@@ -159,16 +178,19 @@ int FirstScreen;		/* TRUE ==> first screen of display */
 Bool PrintErrorMessages = False;	/* controls error messages */
 Bool ShowWelcomeWindow = True;
 static int RedirectError;	/* TRUE ==> another window manager running */
-static int CatchRedirectError();	/* for settting RedirectError */
-static int TwmErrorHandler();	/* for everything else */
+/* for settting RedirectError */
+static int CatchRedirectError(Display *display, XErrorEvent *event);
+/* for everything else */
+static int TwmErrorHandler(Display *display, XErrorEvent *event);
 char Info[INFO_LINES][INFO_SIZE];		/* info strings to print */
-int InfoLines,InfoWidth,InfoHeight;
+int InfoLines;
+unsigned int InfoWidth,InfoHeight;
 char *InitFile = NULL;
-static Window CreateRootWindow ();
-static void DisplayInfo ();
-int InternUsefulAtoms ();
-int InitVariables();
-int CreateFonts();
+static Window CreateRootWindow (int x, int y,
+				unsigned int width, unsigned int height);
+static void DisplayInfo (void);
+void InternUsefulAtoms (void);
+void InitVariables(void);
 
 Cursor	UpperLeftCursor;
 Cursor	TopRightCursor,
@@ -189,11 +211,13 @@ XContext MenuContext;		/* context for all menu windows */
 XContext IconManagerContext;	/* context for all window list windows */
 XContext ScreenContext;		/* context to get screen data */
 XContext ColormapContext;	/* context for colormap operations */
+XContext VirtScreenContext;	/* context for virtual screen */
 
 XClassHint NoClass;		/* for applications with no class */
 
 XGCValues Gcv;
 
+Window captiveroot;
 char *Home;			/* the HOME environment variable */
 int HomeLen;			/* length of Home */
 int ParseError;			/* error parsing the .twmrc file */
@@ -222,11 +246,11 @@ Bool TrapExceptions = True;
 
 unsigned long black, white;
 
-extern void assign_var_savecolor();
-SIGNAL_T Restart();
-SIGNAL_T Crash();
+Bool RestartFlag = 0;
+SIGNAL_T Restart(int signum);
+SIGNAL_T Crash(int signum);
 #ifdef __WAIT_FOR_CHILDS
-  SIGNAL_T ChildExit();
+  SIGNAL_T ChildExit(int signum);
 #endif
 
 extern Atom _XA_WM_WORKSPACESLIST;
@@ -240,22 +264,17 @@ extern Atom _XA_WM_WORKSPACESLIST;
  */
 
 #ifdef VMS
-main(int argc, char **argv)
+int main(int argc, char **argv)
 #else
-main(argc, argv, environ)
-    int argc;
-    char **argv;
-    char **environ;
+int main(int argc, char **argv, char **environ)
 #endif
 {
-    Window root, parent, *children;
+    Window croot, parent, *children;
     unsigned int nchildren;
     int i, j;
-    char *display_name = NULL;
     unsigned long valuemask;	/* mask for create windows */
     XSetWindowAttributes attributes;	/* attributes for create windows */
     int numManaged, firstscrn, lastscrn, scrnum;
-    extern ColormapWindow *CreateColormapWindow();
 #ifdef X11R6
     int zero = 0;
     char *restore_filename = NULL;
@@ -263,10 +282,12 @@ main(argc, argv, environ)
 #endif
     char *welcomefile;
     int  screenmasked;
-    static int rootx = 100;
-    static int rooty = 100;
-    static unsigned int rootw = 1024;
-    static unsigned int rooth = 768;
+    static int crootx = 100;
+    static int crooty = 100;
+    static unsigned int crootw = 1280;
+    static unsigned int crooth =  768;
+/*    static unsigned int crootw = 2880; */
+/*    static unsigned int crooth = 1200; */
     Window capwin = (Window) 0;
     IconRegion *ir;
 
@@ -328,11 +349,19 @@ main(argc, argv, environ)
 		}
 		PrintErrorMessages = True;
 		continue;
+	      case 'c':				/* -cfgchk */
+		if(!strcmp(argv[i],"-cfgchk")) {
+		    cfgchk	= 1;
+		    continue;
+		}
 #ifdef X11R6
-	      case 'c':				/* -clientId */
-		if (++i >= argc) goto usage;
+		if (++i >= argc) goto usage;	/* -clientId */
 		client_id = argv[i];
 		continue;
+#else
+		goto usage;
+#endif
+#ifdef X11R6
 	      case 'r':				/* -restore */
 		if (++i >= argc) goto usage;
 		restore_filename = argv[i];
@@ -386,9 +415,9 @@ main(argc, argv, environ)
       usage:
 	fprintf (stderr, "usage: %s [-display dpy] [-version] [-info]", ProgramName);
 #ifdef USEM4
-	fprintf (stderr, " [-f file] [-s] [-q] [-v] [-W] [-w [wid]] [-k] [-K file] [-n] [-name name]\n");
+	fprintf (stderr, " [-cfgchk] [-f file] [-s] [-q] [-v] [-W] [-w [wid]] [-k] [-K file] [-n] [-name name]\n");
 #else
-	fprintf (stderr, " [-f file] [-s] [-q] [-v] [-W] [-w [wid]] [-name name] \n");
+	fprintf (stderr, " [-cfgchk] [-f file] [-s] [-q] [-v] [-W] [-w [wid]] [-name name] \n");
 #endif
 	exit (1);
     }
@@ -397,7 +426,7 @@ main(argc, argv, environ)
     if (signal (sig, SIG_IGN) != SIG_IGN) (void) signal (sig, action)
 
     newhandler (SIGINT, Done);
-    newhandler (SIGHUP, Restart);
+    signal (SIGHUP, Restart);
     newhandler (SIGQUIT, Done);
     newhandler (SIGTERM, Done);
 #ifdef __WAIT_FOR_CHILDS
@@ -456,6 +485,7 @@ main(argc, argv, environ)
     IconManagerContext = XUniqueContext();
     ScreenContext = XUniqueContext();
     ColormapContext = XUniqueContext();
+    VirtScreenContext = XUniqueContext();
 
     InternUsefulAtoms ();
 
@@ -489,44 +519,43 @@ main(argc, argv, environ)
     FirstScreen = TRUE;
     for (scrnum = firstscrn ; scrnum <= lastscrn; scrnum++)
     {
+        unsigned long attrmask;
 	if (captive) {
 	    XWindowAttributes wa;
 	    if (capwin && XGetWindowAttributes (dpy, capwin, &wa)) {
 		Window junk;
-
-		root  = capwin;
-		rootw = wa.width;
-		rooth = wa.height;
-		XTranslateCoordinates (dpy, capwin, wa.root, 0, 0, &rootx, &rooty, &junk);
+		croot  = capwin;
+		crootw = wa.width;
+		crooth = wa.height;
+		XTranslateCoordinates (dpy, capwin, wa.root, 0, 0, &crootx, &crooty, &junk);
 	    }
 	    else {
-		root = CreateRootWindow (rootx, rooty, rootw, rooth);
+		croot = CreateRootWindow (crootx, crooty, crootw, crooth);
 	    }
+	    captiveroot = croot;
 	}
 	else {
-	    root = RootWindow (dpy, scrnum);
+	    croot  = RootWindow (dpy, scrnum);
+	    crootx = 0;
+	    crooty = 0;
+	    crootw = DisplayWidth  (dpy, scrnum);;
+	    crooth = DisplayHeight (dpy, scrnum);;
 	}
 
         /* Make sure property priority colors is empty */
-        XChangeProperty (dpy, root, _XA_MIT_PRIORITY_COLORS,
+        XChangeProperty (dpy, croot, _XA_MIT_PRIORITY_COLORS,
 			 XA_CARDINAL, 32, PropModeReplace, NULL, 0);
 	XSync(dpy, 0); /* Flush possible previous errors */
 	RedirectError = FALSE;
 	XSetErrorHandler(CatchRedirectError);
-	if (captive) 
-	    XSelectInput(dpy, root,
-		ColormapChangeMask | EnterWindowMask | PropertyChangeMask | 
-		SubstructureRedirectMask | KeyPressMask |
-		ButtonPressMask | ButtonReleaseMask | StructureNotifyMask);
-	else
-	    XSelectInput(dpy, root,
-		ColormapChangeMask | EnterWindowMask | PropertyChangeMask | 
-		SubstructureRedirectMask | KeyPressMask |
-		ButtonPressMask | ButtonReleaseMask);
+	attrmask = ColormapChangeMask | EnterWindowMask | PropertyChangeMask | 
+	  SubstructureRedirectMask | KeyPressMask | ButtonPressMask | ButtonReleaseMask;
+	if (captive) attrmask |= StructureNotifyMask;
+	XSelectInput (dpy, croot, attrmask);
 	XSync(dpy, 0);
 	XSetErrorHandler(TwmErrorHandler);
 
-	if (RedirectError)
+	if (RedirectError && cfgchk==0)
 	{
 	    fprintf (stderr, "%s:  another window manager is already running",
 		     ProgramName);
@@ -542,9 +571,9 @@ main(argc, argv, environ)
 	/* Note:  ScreenInfo struct is calloc'ed to initialize to zero. */
 	Scr = ScreenList[scrnum] = 
 	    (ScreenInfo *) calloc(1, sizeof(ScreenInfo));
-  	if (Scr == NULL)
-  	{
-  	    fprintf (stderr, "%s: unable to allocate memory for ScreenInfo structure for screen %d.\n",
+  	if (Scr == NULL) {
+  	    fprintf (stderr,
+		     "%s: unable to allocate memory for ScreenInfo structure for screen %d.\n",
   		     ProgramName, scrnum);
   	    continue;
   	}
@@ -568,6 +597,7 @@ main(argc, argv, environ)
 	Scr->DontSetInactive = NULL;
 	Scr->AutoSqueeze = NULL;
 	Scr->StartSqueezed = NULL;
+	Scr->AlwaysSqueezeToGravityL = NULL;
 	Scr->MakeTitle = NULL;
 	Scr->AutoRaise = NULL;
 	Scr->AutoLower = NULL;
@@ -598,7 +628,9 @@ main(argc, argv, environ)
 	Scr->HighlightPixmapName = NULL;
 	Scr->Workspaces = (MenuRoot*) 0;
 	Scr->IconMenuDontShow = NULL;
-
+	Scr->VirtualScreens = NULL;
+	Scr->CaptiveRoot = captiveroot;
+	Scr->IgnoreTransientL = NULL;
 
 	/* remember to put an initialization in InitVariables also
 	 */
@@ -606,58 +638,51 @@ main(argc, argv, environ)
 	Scr->screen = scrnum;
 	Scr->d_depth = DefaultDepth(dpy, scrnum);
 	Scr->d_visual = DefaultVisual(dpy, scrnum);
-	Scr->Root = root;
+	Scr->RealRoot = RootWindow (dpy, scrnum);
+	Scr->Root = croot;
 	XSaveContext (dpy, Scr->Root, ScreenContext, (XPointer) Scr);
+
 	if (captive) {
 	    AddToCaptiveList ();
 	    if (captivename) {
-		XSetStandardProperties (dpy, root, captivename, captivename,
-				None, NULL, 0, NULL);
+		XSetStandardProperties (dpy, croot, captivename, captivename, None, NULL, 0, NULL);
 	    }
 	} else {
 	    captivename = "Root";
 	}
 	Scr->TwmRoot.cmaps.number_cwins = 1;
-	Scr->TwmRoot.cmaps.cwins =
-		(ColormapWindow **) malloc(sizeof(ColormapWindow *));
-	Scr->TwmRoot.cmaps.cwins[0] =
-		CreateColormapWindow(Scr->Root, True, False);
+	Scr->TwmRoot.cmaps.cwins = (ColormapWindow **) malloc(sizeof(ColormapWindow *));
+	Scr->TwmRoot.cmaps.cwins[0] = CreateColormapWindow(Scr->Root, True, False);
 	Scr->TwmRoot.cmaps.cwins[0]->visibility = VisibilityPartiallyObscured;
 
 	Scr->cmapInfo.cmaps = NULL;
-	Scr->cmapInfo.maxCmaps =
-		MaxCmapsOfScreen(ScreenOfDisplay(dpy, Scr->screen));
+	Scr->cmapInfo.maxCmaps = MaxCmapsOfScreen(ScreenOfDisplay(dpy, Scr->screen));
 	Scr->cmapInfo.root_pushes = 0;
 	InstallWindowColormaps(0, &Scr->TwmRoot);
 
-	Scr->StdCmapInfo.head = Scr->StdCmapInfo.tail = 
-	  Scr->StdCmapInfo.mru = NULL;
+	Scr->StdCmapInfo.head = Scr->StdCmapInfo.tail =  Scr->StdCmapInfo.mru = NULL;
 	Scr->StdCmapInfo.mruindex = 0;
 	LocateStandardColormaps();
 
-	Scr->TBInfo.nleft = Scr->TBInfo.nright = 0;
-	Scr->TBInfo.head = NULL;
-	Scr->TBInfo.border = -100; /* trick to have different default value if ThreeDTitles
-					is set or not */
-	Scr->TBInfo.width = 0;
-	Scr->TBInfo.leftx = 0;
+	Scr->TBInfo.nleft  = Scr->TBInfo.nright = 0;
+	Scr->TBInfo.head   = NULL;
+	Scr->TBInfo.border = -100; /* trick to have different default value if ThreeDTitles */
+	Scr->TBInfo.width  = 0;	   /* is set or not */
+	Scr->TBInfo.leftx  = 0;
 	Scr->TBInfo.titlex = 0;
 
-	if (captive) {
-	    Scr->MyDisplayX      = rootx;
-	    Scr->MyDisplayY      = rooty;
-	    Scr->MyDisplayWidth  = rootw;
-	    Scr->MyDisplayHeight = rooth;
-	}
-	else {
-	    Scr->MyDisplayX      = 0;
-	    Scr->MyDisplayY      = 0;
-	    Scr->MyDisplayWidth  = DisplayWidth(dpy, scrnum);
-	    Scr->MyDisplayHeight = DisplayHeight(dpy, scrnum);
-	}
+	Scr->rootx  = crootx;
+	Scr->rooty  = crooty;
+ 	Scr->rootw  = crootw;
+	Scr->rooth  = crooth;
 
-	Scr->MaxWindowWidth = 32767 - Scr->MyDisplayWidth;
-	Scr->MaxWindowHeight = 32767 - Scr->MyDisplayHeight;
+	Scr->crootx = crootx;
+	Scr->crooty = crooty;
+	Scr->crootw = crootw;
+	Scr->crooth = crooth;
+
+	Scr->MaxWindowWidth  = 32767 - Scr->rootw;
+	Scr->MaxWindowHeight = 32767 - Scr->rooth;
 
 	Scr->XORvalue = (((unsigned long) 1) << Scr->d_depth) - 1;
 
@@ -717,14 +742,32 @@ main(argc, argv, environ)
 	InitMenus();
 	InitWorkSpaceManager ();
 
+	InitVirtualScreens (Scr);
+	ConfigureWorkSpaceManager ();
+
 	/* Parse it once for each screen. */
-	ParseTwmrc(InitFile);
+	if(cfgchk) {
+	  if(ParseTwmrc(InitFile)==0) {
+	        /* Error return */
+	        fprintf (stderr, "Errors found\n");
+		exit(1);
+	    } else {
+	        fprintf (stderr, "No errors found\n");
+		exit(0);
+	    }
+	}
+	else
+	  {
+	    ParseTwmrc(InitFile);
+	  }
+
 	if (ShowWelcomeWindow && ! screenmasked) MaskScreen (NULL);
 	if (Scr->ClickToFocus) {
 	    Scr->FocusRoot  = FALSE;
 	    Scr->TitleFocus = FALSE;
 	}
-	ConfigureWorkSpaceManager ();
+
+
 
 	if (Scr->use3Dtitles) {
 	    if (Scr->FramePadding  == -100) Scr->FramePadding  = 0;
@@ -785,8 +828,11 @@ main(argc, argv, environ)
 	CreateWorkSpaceManager ();
 	MakeWorkspacesMenu ();
 	createWindowBoxes ();
+#ifdef GNOME
+	InitGnome ();
+#endif /* GNOME */
 
-	XQueryTree(dpy, Scr->Root, &root, &parent, &children, &nchildren);
+	XQueryTree(dpy, Scr->Root, &croot, &parent, &children, &nchildren);
 	/*
 	 * weed out icon windows
 	 */
@@ -821,14 +867,17 @@ main(argc, argv, environ)
 	}
 	if (Scr->ShowWorkspaceManager && Scr->workSpaceManagerActive)
 	{
+	    virtualScreen *vs;
 	    if (Scr->WindowMask) XRaiseWindow (dpy, Scr->WindowMask);
-	    SetMapStateProp (Scr->workSpaceMgr.workspaceWindow.twm_win, NormalState);
-	    XMapWindow (dpy, Scr->workSpaceMgr.workspaceWindow.twm_win->frame);
-	    if (Scr->workSpaceMgr.workspaceWindow.twm_win->StartSqueezed)
-		Squeeze (Scr->workSpaceMgr.workspaceWindow.twm_win);
-	    else
-		XMapWindow (dpy, Scr->workSpaceMgr.workspaceWindow.w);
-	    Scr->workSpaceMgr.workspaceWindow.twm_win->mapped = TRUE;
+	    for (vs = Scr->vScreenList; vs != NULL; vs = vs->next) {
+		SetMapStateProp (vs->wsw->twm_win, NormalState);
+		XMapWindow (dpy, vs->wsw->twm_win->frame);
+		if (vs->wsw->twm_win->StartSqueezed)
+		  Squeeze (vs->wsw->twm_win);
+		else
+		  XMapWindow (dpy, vs->wsw->w);
+		vs->wsw->twm_win->mapped = TRUE;
+	    }
 	}
 	
 	if (!Scr->BeNiceToColormap) GetShadeColors (&Scr->DefaultC);
@@ -862,8 +911,8 @@ main(argc, argv, environ)
 	{
 	    int sx, sy;
 	    if (Scr->CenterFeedbackWindow) {
-		sx = (Scr->MyDisplayWidth  / 2) - (Scr->SizeStringWidth / 2);
-		sy = (Scr->MyDisplayHeight / 2) - ((Scr->SizeFont.height + SIZE_VINDENT*2) / 2);
+		sx = (Scr->rootw / 2) - (Scr->SizeStringWidth / 2);
+		sy = (Scr->rooth / 2) - ((Scr->SizeFont.height + SIZE_VINDENT*2) / 2);
 		attributes.save_under = True;
 		valuemask |= CWSaveUnder;
 	    } else {
@@ -879,8 +928,8 @@ main(argc, argv, environ)
 					 (Visual *) CopyFromParent,
 					 valuemask, &attributes);
 	}
-	Scr->ShapeWindow = XCreateSimpleWindow (dpy, Scr->Root, 0, 0, Scr->MyDisplayWidth,
-				Scr->MyDisplayHeight, 0, 0, 0);
+	Scr->ShapeWindow = XCreateSimpleWindow (dpy, Scr->Root, 0, 0,
+						Scr->rootw, Scr->rooth, 0, 0, 0);
 
 	XUngrabServer(dpy);
 	if (ShowWelcomeWindow) UnmaskScreen ();
@@ -918,7 +967,7 @@ main(argc, argv, environ)
  ***********************************************************************
  */
 
-InitVariables()
+void InitVariables(void)
 {
     FreeList(&Scr->BorderColorL);
     FreeList(&Scr->IconBorderColorL);
@@ -959,8 +1008,10 @@ InitVariables()
     FreeList(&Scr->DontSetInactive);
     FreeList(&Scr->AutoSqueeze);
     FreeList(&Scr->StartSqueezed);
+    FreeList(&Scr->AlwaysSqueezeToGravityL);
     FreeList(&Scr->IconMenuDontShow);
-
+    FreeList(&Scr->VirtualScreens);
+    FreeList(&Scr->IgnoreTransientL);
 
     NewFontCursor(&Scr->FrameCursor, "top_left_arrow");
     NewFontCursor(&Scr->TitleCursor, "top_left_arrow");
@@ -1047,7 +1098,7 @@ InitVariables()
     Scr->AutoOccupy = FALSE;
     Scr->TransientHasOccupation = FALSE;
     Scr->DontPaintRootWindow = FALSE;
-    Scr->IconManagerDontShow =FALSE;
+    Scr->IconManagerDontShow = FALSE;
     Scr->BackingStore = TRUE;
     Scr->SaveUnder = TRUE;
     Scr->RandomPlacement = RP_OFF;
@@ -1094,8 +1145,8 @@ InitVariables()
     Scr->IconRegionJustification = J_CENTER;
     Scr->IconRegionAlignement = J_CENTER;
     Scr->TitleJustification = J_LEFT;
-    Scr->SmartIconify = FALSE;
-    Scr->MaxIconTitleWidth = Scr->MyDisplayWidth;
+    Scr->IconifyStyle = ICONIFY_NORMAL;
+    Scr->MaxIconTitleWidth = Scr->rootw;
     Scr->ReallyMoveInWorkspaceManager = FALSE;
     Scr->ShowWinWhenMovingInWmgr = FALSE;
     Scr->ReverseCurrentWorkspace = FALSE;
@@ -1104,7 +1155,7 @@ InitVariables()
     Scr->YMoveGrid = 1;
     Scr->FastServer = True;
     Scr->CenterFeedbackWindow = False;
-    Scr->SchrinkIconTitles = False;
+    Scr->ShrinkIconTitles = False;
     Scr->AutoRaiseIcons = False;
     Scr->AutoFocusToTransients = False; /* kai */
     Scr->use3Diconborders = False;
@@ -1116,6 +1167,8 @@ InitVariables()
     Scr->IgnoreModifier = 0;
     Scr->IgnoreCaseInMenuSelection = False;
     Scr->PackNewWindows = False;
+    Scr->AlwaysSqueezeToGravity = FALSE;
+    Scr->NoWarpToMenuTitle = FALSE;
 
     Scr->BorderTop    = 0;
     Scr->BorderBottom = 0;
@@ -1140,6 +1193,8 @@ InitVariables()
     Scr->IconManagerFont.basename = DEFAULT_NICE_FONT;
     Scr->DefaultFont.font_set = NULL;
     Scr->DefaultFont.basename = DEFAULT_FAST_FONT;
+    Scr->workSpaceMgr.windowFont.font_set = NULL;
+    Scr->workSpaceMgr.windowFont.basename = DEFAULT_FAST_FONT;
 #else    
 #   define DEFAULT_NICE_FONT "variable"
 #   define DEFAULT_FAST_FONT "fixed"
@@ -1156,12 +1211,14 @@ InitVariables()
     Scr->IconManagerFont.name = DEFAULT_NICE_FONT;
     Scr->DefaultFont.font = NULL;
     Scr->DefaultFont.name = DEFAULT_FAST_FONT;
-#endif    
+    Scr->workSpaceMgr.windowFont.font = NULL;
+    Scr->workSpaceMgr.windowFont.name = DEFAULT_FAST_FONT;
+#endif
 
 }
 
 
-CreateFonts ()
+void CreateFonts (void)
 {
     GetFont(&Scr->TitleBarFont);
     GetFont(&Scr->MenuFont);
@@ -1169,19 +1226,19 @@ CreateFonts ()
     GetFont(&Scr->SizeFont);
     GetFont(&Scr->IconManagerFont);
     GetFont(&Scr->DefaultFont);
-    GetFont(&Scr->workSpaceMgr.workspaceWindow.windowFont);
+    GetFont(&Scr->workSpaceMgr.windowFont);
     Scr->HaveFonts = TRUE;
 }
 
 
-RestoreWithdrawnLocation (tmp)
-    TwmWindow *tmp;
+void RestoreWithdrawnLocation (TwmWindow *tmp);
+void RestoreWithdrawnLocation (TwmWindow *tmp)
 {
     int gravx, gravy;
     unsigned int bw, mask;
     XWindowChanges xwc;
 
-    if (tmp->UnmapByMovingFarAway && !VISIBLE(tmp)) {
+    if (tmp->UnmapByMovingFarAway && !visible(tmp)) {
 	XMoveWindow (dpy, tmp->frame, tmp->frame_x, tmp->frame_y);
     }
     if (tmp->squeezed) Squeeze (tmp);
@@ -1218,12 +1275,18 @@ RestoreWithdrawnLocation (tmp)
 	    mask |= CWBorderWidth;
 	}
 
-	if (tmp->winbox && tmp->winbox->twmwin) {
-	  GetGravityOffsets (tmp->winbox->twmwin, &gravx, &gravy);
-	  xwc.x -= gravx * tmp->winbox->twmwin->frame_bw3D;
-	  xwc.y -= gravy * tmp->winbox->twmwin->frame_bw3D;
+	if (tmp->vs) {
+	  xwc.x += tmp->vs->x;
+	  xwc.y += tmp->vs->y;
 	}
 
+	if (tmp->winbox && tmp->winbox->twmwin && tmp->frame) {
+	  int xbox, ybox;
+	  if (XGetGeometry (dpy, tmp->frame, &JunkRoot, &xbox, &ybox, 
+			    &JunkWidth, &JunkHeight, &bw, &JunkDepth)) {
+	    XReparentWindow  (dpy, tmp->frame, Scr->Root, xbox, ybox);
+	  }
+	}
 	XConfigureWindow (dpy, tmp->w, mask, &xwc);
 
 	if (tmp->wmhints && (tmp->wmhints->flags & IconWindowHint)) {
@@ -1254,12 +1317,11 @@ RestoreWithdrawnLocation (tmp)
  ***********************************************************************
  */
 
-void Reborder (time)
-Time time;
+void Reborder (Time time)
 {
     TwmWindow *tmp;			/* temp twm window structure */
     int scrnum;
-    ScreenInfo *savedScreen;	        /* Its better to avoid coredumps */
+    ScreenInfo *savedScreen;		/* Its better to avoid coredumps */
 
     /* put a border back around all windows */
 
@@ -1282,7 +1344,7 @@ Time time;
     SetFocus ((TwmWindow*)NULL, time);
 }
 
-SIGNAL_T Done()
+SIGNAL_T Done(int signum)
 {
 #ifdef SOUNDS
     play_exit_sound();
@@ -1304,7 +1366,7 @@ SIGNAL_T Done()
 #endif
 }
 
-SIGNAL_T Crash ()
+SIGNAL_T Crash (int signum)
 {
     Reborder (CurrentTime);
     XDeleteProperty (dpy, Scr->Root, _XA_WM_WORKSPACESLIST);
@@ -1315,27 +1377,43 @@ SIGNAL_T Crash ()
     fprintf (stderr, "If a core file was generated in your directory,\n");
     fprintf (stderr, "can you please try extract the stack trace,\n");
     fprintf (stderr, "and mail the results, and a description of what you were doing,\n");
-    fprintf (stderr, "to Claude.Lecommandeur@epfl.ch.  Thank you for your support.\n");
+    fprintf (stderr, "to ctwm-bugs@free.lp.se.  Thank you for your support.\n");
     fprintf (stderr, "...exiting ctwm now.\n\n");
 
     abort ();
 }
 
 
-SIGNAL_T Restart()
+SIGNAL_T Restart(int signum)
 {
+    fprintf (stderr, "%s:  setting restart flag\n", ProgramName);
+    RestartFlag = 1;
+}
+
+void DoRestart(Time t)
+{
+#ifdef X11R6
+    extern SmcConn smcConn;
+#endif
+    RestartFlag = 0;
+
     StopAnimation ();
     XSync (dpy, 0);
-    Reborder (CurrentTime);
+    Reborder (t);
     XSync (dpy, 0);
+
+#ifdef X11R6
+    if (smcConn) SmcCloseConnection (smcConn, 0, NULL);
+#endif /* X11R6 */
+
+    fprintf (stderr, "%s:  restarting:  %s\n",
+	     ProgramName, *Argv);
 #ifdef VMS
-    fprintf (stderr, "%s:  restart capabilities not yet supported\n",
-	     ProgramName);
+    exit (1);			/* Trust CTWM.COM  /Richard Levitte */
 #else
     execvp(*Argv, Argv);
 #endif
     fprintf (stderr, "%s:  unable to restart:  %s\n", ProgramName, *Argv);
-    exit (1);
 }
 
 #ifdef __WAIT_FOR_CHILDS
@@ -1364,9 +1442,7 @@ ChildExit (int signum)
 Bool ErrorOccurred = False;
 XErrorEvent LastErrorEvent;
 
-static int TwmErrorHandler(display, event)
-    Display *display;
-    XErrorEvent *event;
+static int TwmErrorHandler(Display *display, XErrorEvent *event)
 {
     LastErrorEvent = *event;
     ErrorOccurred = True;
@@ -1381,9 +1457,7 @@ static int TwmErrorHandler(display, event)
 
 
 /* ARGSUSED*/
-static int CatchRedirectError(display, event)
-    Display *display;
-    XErrorEvent *event;
+static int CatchRedirectError(Display *display, XErrorEvent *event)
 {
     RedirectError = TRUE;
     LastErrorEvent = *event;
@@ -1406,7 +1480,7 @@ Atom _XA_WM_CLIENT_MACHINE;
   Atom _XA_WM_WINDOW_ROLE;
 #endif
 
-InternUsefulAtoms ()
+void InternUsefulAtoms (void)
 {
     /* 
      * Create priority colors if necessary.
@@ -1427,9 +1501,8 @@ InternUsefulAtoms ()
 #endif
 }
 
-static Window CreateRootWindow (x, y, width, height)
-int		x, y;
-unsigned int	width, height;
+static Window CreateRootWindow (int x, int y,
+				unsigned int width, unsigned int height)
 {
     int		scrnum;
     Window	ret;
@@ -1449,11 +1522,12 @@ unsigned int	width, height;
     _XA_WM_CTWM_ROOT = XInternAtom (dpy, "WM_CTWM_ROOT", False);
     XChangeProperty (dpy, ret, _XA_WM_CTWM_ROOT, XA_WINDOW, 32, 
 		     PropModeReplace, (unsigned char *) &ret, 4);
+    XSelectInput (dpy, ret, StructureNotifyMask);
     XMapWindow (dpy, ret);
     return (ret);
 }
 
-static void DisplayInfo () {
+static void DisplayInfo (void) {
     (void) printf ("Twm version:  %s\n", Version);
     (void) printf ("Compile time options :");
 #ifdef XPM
