@@ -106,7 +106,7 @@ int iconifybox_height = siconify_height;
  ***********************************************************************
  */
 
-void CreateIconManagers()
+void CreateIconManagers(void)
 {
     IconMgr *p, *q;
     int mask;
@@ -114,7 +114,7 @@ void CreateIconManagers()
     char str1[100];
     Pixel background;
     char *icon_name;
-    WorkSpaceList *wlist;
+    WorkSpace    *ws;
     XWMHints	  wmhints;
     XSizeHints	  sizehints;
     int		  gravity;
@@ -130,7 +130,7 @@ void CreateIconManagers()
 	    (char *)siconify_bits, siconify_width, siconify_height, 1, 0, 1);
     }
 
-    wlist = Scr->workSpaceMgr.workSpaceList;
+    ws = Scr->workSpaceMgr.workSpaceList;
     for (q = Scr->iconmgr; q != NULL; q = q->nextv) {
       for (p = q; p != NULL; p = p->next)
       {
@@ -149,13 +149,13 @@ void CreateIconManagers()
 		(Scr->ThreeDBorderWidth ? Scr->ThreeDBorderWidth : Scr->BorderWidth);
 
 	if (mask & XNegative) {
-	    JunkX += Scr->MyDisplayWidth - p->width - 2 * bw;
+	    JunkX += Scr->rootw - p->width - 2 * bw;
 	    gravity = (mask & YNegative) ? SouthEastGravity : NorthEastGravity;
 	} else {
 	    gravity = (mask & YNegative) ? SouthWestGravity : NorthWestGravity;
 	}
 	if (mask & YNegative)
-	    JunkY += Scr->MyDisplayHeight - p->height - 2 * bw;
+	    JunkY += Scr->rooth - p->height - 2 * bw;
 
 	background = Scr->IconManagerC.back;
 	GetColorFromList(Scr->IconManagerBL, p->name, (XClassHint *)NULL,
@@ -169,12 +169,16 @@ void CreateIconManagers()
 
 	XSetStandardProperties(dpy, p->w, str, icon_name, None, NULL, 0, NULL);
 
-	Scr->workSpaceMgr.activeWSPC = wlist;
+	/* Scr->workSpaceMgr.activeWSPC = ws; */
 	wmhints.initial_state = NormalState;
 	wmhints.input         = True;
 	wmhints.flags         = InputHint | StateHint;
 	XSetWMHints (dpy, p->w, &wmhints);
 	p->twm_win = AddWindow(p->w, TRUE, p);
+	if (ws)
+	  p->twm_win->occupation = 1 << ws->number;
+	else
+	  p->twm_win->occupation = 1;
 
 	sizehints.flags       = PWinGravity;
 	sizehints.win_gravity = gravity;
@@ -194,9 +198,11 @@ void CreateIconManagers()
 	    p->twm_win->isicon = TRUE;
 	}
       }
-      if (wlist != NULL) wlist = wlist->next;
+      if (ws != NULL) ws = ws->next;
+    }
+    for (p = Scr->iconmgr; p != NULL; p = p->next) {
+      p->twm_win->vs = Scr->vScreenList;
     }
-    Scr->workSpaceMgr.activeWSPC = Scr->workSpaceMgr.workSpaceList;
     if (Scr->workSpaceManagerActive)
 	Scr->workSpaceMgr.workSpaceList->iconmgr = Scr->iconmgr;
 
@@ -223,11 +229,8 @@ void CreateIconManagers()
  ***********************************************************************
  */
 
-IconMgr *AllocateIconManager(name, icon_name, geom, columns)
-    char *name;
-    char *geom;
-    char *icon_name;
-    int columns;
+IconMgr *AllocateIconManager(char *name, char *icon_name, char *geom,
+			     int columns)
 {
     IconMgr *p;
 
@@ -291,8 +294,7 @@ IconMgr *AllocateIconManager(name, icon_name, geom, columns)
  ***********************************************************************
  */
 
-void MoveIconManager(dir)
-    int dir;
+void MoveIconManager(int dir)
 {
     IconMgr *ip;
     WList *tmp = NULL;
@@ -418,14 +420,14 @@ void MoveIconManager(dir)
  ***********************************************************************
  */
 
-void MoveMappedIconManager(dir)
-    int dir;
+void MoveMappedIconManager(int dir)
 {
     IconMgr *ip;
     WList *tmp = NULL;
     WList *orig = NULL;
     int got_it;
 
+    if (!Current) Current = Active; 
     if (!Current) return;
 
     ip = Current->iconmgr;
@@ -471,10 +473,8 @@ void MoveMappedIconManager(dir)
 	XWarpPointer(dpy, None, tmp->icon, 0,0,0,0, 5, 5);
     } else {
 	if (tmp->twm->title_height) {
-	    int tbx = Scr->TBInfo.titlex;
-	    int x = tmp->twm->highlightxr;
 	    XWarpPointer (dpy, None, tmp->twm->title_w, 0, 0, 0, 0,
-			  tbx + (x - tbx) / 2,
+			  tmp->twm->title_width / 2, 
 			  Scr->TitleHeight / 4);
 	} else {
 	    XWarpPointer (dpy, None, tmp->twm->w, 0, 0, 0, 0, 5, 5);
@@ -496,8 +496,7 @@ void MoveMappedIconManager(dir)
  ***********************************************************************
  */
 
-void JumpIconManager(dir)
-    register int dir;
+void JumpIconManager(register int dir)
 {
     IconMgr *ip, *tmp_ip = NULL;
     int got_it = FALSE;
@@ -565,8 +564,7 @@ void JumpIconManager(dir)
  ***********************************************************************
  */
 
-WList *AddIconManager(tmp_win)
-    TwmWindow *tmp_win;
+WList *AddIconManager(TwmWindow *tmp_win)
 {
     WList *tmp, *old;
     int h, offs;
@@ -575,8 +573,8 @@ WList *AddIconManager(tmp_win)
     IconMgr *ip;
 
     if (tmp_win->iconmgr || tmp_win->transient || Scr->NoIconManagers ||
-	tmp_win->w == Scr->workSpaceMgr.workspaceWindow.w ||
-	tmp_win->w == Scr->workSpaceMgr.occupyWindow.w)
+	tmp_win->wspmgr ||
+	tmp_win->w == Scr->workSpaceMgr.occupyWindow->w)
 	return NULL;
 
     if (LookInList(Scr->IconMgrNoShow, tmp_win->full_name, &tmp_win->class))
@@ -585,16 +583,20 @@ WList *AddIconManager(tmp_win)
 	!LookInList(Scr->IconMgrShow, tmp_win->full_name, &tmp_win->class))
 	return NULL;
     if ((ip = (IconMgr *)LookInList(Scr->IconMgrs, tmp_win->full_name,
-	    &tmp_win->class)) == NULL)
+	    &tmp_win->class)) == NULL) {
 	if (Scr->workSpaceManagerActive)
 	    ip = Scr->workSpaceMgr.workSpaceList->iconmgr;
 	else
 	    ip = Scr->iconmgr;
+    }
 
   tmp = NULL;
   old = tmp_win->list;
   while (ip != NULL) {
-    if ((tmp_win->occupation & ip->twm_win->occupation) == 0) {ip = ip->nextv; continue;}
+    if ((tmp_win->occupation & ip->twm_win->occupation) == 0) {
+      ip = ip->nextv;
+      continue;
+    }
     tmp = (WList *) malloc(sizeof(WList));
     tmp->iconmgr = ip;
     tmp->next = NULL;
@@ -635,8 +637,9 @@ WList *AddIconManager(tmp_win)
     attributes.background_pixel = tmp->cp.back;
     attributes.border_pixel = tmp->cp.back;
     attributes.event_mask = (KeyPressMask | ButtonPressMask |
-			     ButtonReleaseMask | ExposureMask |
-			     EnterWindowMask | LeaveWindowMask);
+			     ButtonReleaseMask | ExposureMask);
+    if (Scr->IconManagerFocus)
+	attributes.event_mask |= (EnterWindowMask | LeaveWindowMask);
     attributes.cursor = Scr->IconMgrCursor;
     tmp->w = XCreateWindow (dpy, ip->w, 0, 0, (unsigned int) 1, 
 			    (unsigned int) h, (unsigned int) 0, 
@@ -646,8 +649,7 @@ WList *AddIconManager(tmp_win)
     valuemask = (CWBackPixel | CWBorderPixel | CWEventMask | CWCursor);
     attributes.background_pixel = tmp->cp.back;
     attributes.border_pixel = Scr->Black;
-    attributes.event_mask = (ButtonReleaseMask| ButtonPressMask |
-			     ExposureMask);
+    attributes.event_mask = (ButtonReleaseMask| ButtonPressMask | ExposureMask);
     attributes.cursor = Scr->ButtonCursor;
     offs = Scr->use3Diconmanagers ? Scr->IconManagerShadowDepth : 2;
     tmp->icon = XCreateWindow (dpy, tmp->w, offs + 3,
@@ -672,7 +674,7 @@ WList *AddIconManager(tmp_win)
 
     if (!ip->twm_win->isicon)
     {
-	if (OCCUPY (ip->twm_win, Scr->workSpaceMgr.activeWSPC)) {
+	if (visible (ip->twm_win)) {
 	    SetMapStateProp (ip->twm_win, NormalState);
 	    XMapWindow (dpy, ip->w);
 	    XMapWindow (dpy, ip->twm_win->frame);
@@ -685,11 +687,11 @@ WList *AddIconManager(tmp_win)
   }
     if (tmp == NULL) return NULL;
     tmp_win->list = tmp;
-    if (! OCCUPY (tmp->iconmgr->twm_win, Scr->workSpaceMgr.activeWSPC)) {
+    if (! visible (tmp->iconmgr->twm_win)) {
 	old = tmp;
 	tmp = tmp->nextv;
 	while (tmp != NULL) {
-	    if (OCCUPY (tmp->iconmgr->twm_win, Scr->workSpaceMgr.activeWSPC)) break;
+	    if (visible (tmp->iconmgr->twm_win)) break;
 	    old = tmp;
 	    tmp = tmp->nextv;
 	}
@@ -715,14 +717,10 @@ WList *AddIconManager(tmp_win)
  ***********************************************************************
  */
 
-void InsertInIconManager(ip, tmp, tmp_win)
-    IconMgr *ip;
-    WList *tmp;
-    TwmWindow *tmp_win;
+void InsertInIconManager(IconMgr *ip, WList *tmp, TwmWindow *tmp_win)
 {
     WList *tmp1;
     int added;
-    int (*compar)() = (Scr->CaseSensitive ? strcmp : XmuCompareISOLatin1);
 
     added = FALSE;
     if (ip->first == NULL)
@@ -736,7 +734,12 @@ void InsertInIconManager(ip, tmp, tmp_win)
     {
 	for (tmp1 = ip->first; tmp1 != NULL; tmp1 = tmp1->next)
 	{
-	    if ((*compar)(tmp_win->icon_name, tmp1->twm->icon_name) < 0)
+	    int compresult;
+	    if (Scr->CaseSensitive)
+		compresult = strcmp(tmp_win->icon_name,tmp1->twm->icon_name);
+	    else
+		compresult = XmuCompareISOLatin1(tmp_win->icon_name,tmp1->twm->icon_name);
+	    if (compresult < 0)
 	    {
 		tmp->next = tmp1;
 		tmp->prev = tmp1->prev;
@@ -759,9 +762,7 @@ void InsertInIconManager(ip, tmp, tmp_win)
     }
 }
 
-void RemoveFromIconManager(ip, tmp)
-    IconMgr *ip;
-    WList *tmp;
+void RemoveFromIconManager(IconMgr *ip, WList *tmp)
 {
     if (tmp->prev == NULL)
 	ip->first = tmp->next;
@@ -772,7 +773,10 @@ void RemoveFromIconManager(ip, tmp)
 	ip->last = tmp->prev;
     else
 	tmp->next->prev = tmp->prev;
-    if (Current == tmp) Current = NULL;
+
+    /* pebl: If the list was the current and tmp was the last in the list
+       reset current list */
+    if (Current == tmp) Current = ip->first;
 }
 
 /***********************************************************************
@@ -786,8 +790,7 @@ void RemoveFromIconManager(ip, tmp)
  ***********************************************************************
  */
 
-void RemoveIconManager(tmp_win)
-    TwmWindow *tmp_win;
+void RemoveIconManager(TwmWindow *tmp_win)
 {
     IconMgr *ip;
     WList *tmp, *tmp1, *save;
@@ -832,14 +835,12 @@ void RemoveIconManager(tmp_win)
   }
 }
 
-void CurrentIconManagerEntry (current)
-WList *current;
+void CurrentIconManagerEntry (WList *current)
 {
     Current = current;
 }
 
-void ActiveIconManager(active)
-    WList *active;
+void ActiveIconManager(WList *active)
 {
     active->active = TRUE;
     Active = active;
@@ -848,16 +849,13 @@ void ActiveIconManager(active)
     DrawIconManagerBorder(active, False);
 }
 
-void NotActiveIconManager(active)
-    WList *active;
+void NotActiveIconManager(WList *active)
 {
     active->active = FALSE;
     DrawIconManagerBorder(active, False);
 }
 
-void DrawIconManagerBorder(tmp, fill)
-    WList *tmp;
-    int fill;
+void DrawIconManagerBorder(WList *tmp, int fill)
 {
     if (Scr->use3Diconmanagers) {
 	if (tmp->active && Scr->Highlight)
@@ -892,12 +890,10 @@ void DrawIconManagerBorder(tmp, fill)
  ***********************************************************************
  */
 
-void SortIconManager(ip)
-    IconMgr *ip;
+void SortIconManager(IconMgr *ip)
 {
     WList *tmp1, *tmp2;
     int done;
-    int (*compar)() = (Scr->CaseSensitive ? strcmp : XmuCompareISOLatin1);
 
     if (ip == NULL)
 	ip = Active->iconmgr;
@@ -907,12 +903,17 @@ void SortIconManager(ip)
     {
 	for (tmp1 = ip->first; tmp1 != NULL; tmp1 = tmp1->next)
 	{
+	    int compresult;
 	    if ((tmp2 = tmp1->next) == NULL)
 	    {
 		done = TRUE;
 		break;
 	    }
-	    if ((*compar)(tmp1->twm->icon_name, tmp2->twm->icon_name) > 0)
+	    if (Scr->CaseSensitive)
+		compresult = strcmp(tmp1->twm->icon_name,tmp2->twm->icon_name);
+	    else
+		compresult = XmuCompareISOLatin1(tmp1->twm->icon_name,tmp2->twm->icon_name);
+	    if (compresult > 0)
 	    {
 		/* take it out and put it back in */
 		RemoveFromIconManager(ip, tmp2);
@@ -937,8 +938,7 @@ void SortIconManager(ip)
  ***********************************************************************
  */
 
-void PackIconManager(ip)
-    IconMgr *ip;
+void PackIconManager(IconMgr *ip)
 {
     int newwidth, i, row, col, maxcol,  colinc, rowinc, wheight, wwidth;
     int new_x, new_y;
