@@ -69,20 +69,16 @@
 #include "parse.h"
 #include "util.h"
 
-extern int twmrc_error_prefix();
+extern void twmrc_error_prefix(void);
 extern Bool AnimationPending;
 extern Bool AnimationActive;
 extern Bool MaybeAnimate;
 
 #define iconWidth(w)	(w->icon->border_width * 2 + \
-			Scr->SchrinkIconTitles ? w->icon->width : w->icon->w_width)
+			Scr->ShrinkIconTitles ? w->icon->width : w->icon->w_width)
 #define iconHeight(w)	(w->icon->border_width * 2 + w->icon->w_height)
 
-static
-splitEntry (ie, grav1, grav2, w, h)
-    IconEntry	*ie;
-    int		grav1, grav2;
-    int		w, h;
+static void splitEntry (IconEntry *ie, int grav1, int grav2, int w, int h)
 {
     IconEntry	*new;
     int		save;
@@ -135,15 +131,13 @@ splitEntry (ie, grav1, grav2, w, h)
     }
 }
 
-roundUp (v, multiple)
+static int roundUp (int v, int multiple)
 {
     return ((v + multiple - 1) / multiple) * multiple;
 }
 
-static void PlaceIcon(tmp_win, def_x, def_y, final_x, final_y)
-TwmWindow *tmp_win;
-int def_x, def_y;
-int *final_x, *final_y;
+static void PlaceIcon(TwmWindow *tmp_win, int def_x, int def_y,
+		      int *final_x, int *final_y)
 {
     IconRegion	*ir, *oldir;
     IconEntry	*ie;
@@ -235,17 +229,14 @@ int *final_x, *final_y;
 	tmp_win->icon->ir = (IconRegion*)0;
 	return;
     }
-    if (Scr->SchrinkIconTitles && tmp_win->icon->has_title) {
+    if (Scr->ShrinkIconTitles && tmp_win->icon->has_title) {
 	*final_x -= GetIconOffset (tmp_win->icon);
 	if (tmp_win->icon->ir != oldir) ReshapeIcon (tmp_win->icon);
     }
     return;
 }
 
-static IconEntry *
-FindIconEntry (tmp_win, irp)
-    TwmWindow   *tmp_win;
-    IconRegion	**irp;
+static IconEntry *FindIconEntry (TwmWindow *tmp_win, IconRegion **irp)
 {
     IconRegion	*ir;
     IconEntry	*ie;
@@ -261,8 +252,7 @@ FindIconEntry (tmp_win, irp)
     return 0;
 }
 
-IconUp (tmp_win)
-    TwmWindow   *tmp_win;
+int IconUp (TwmWindow *tmp_win)
 {
     int		x, y;
     int		defx, defy;
@@ -303,10 +293,7 @@ IconUp (tmp_win)
     return (0);
 }
 
-static IconEntry *
-prevIconEntry (ie, ir)
-    IconEntry	*ie;
-    IconRegion	*ir;
+static IconEntry *prevIconEntry (IconEntry *ie, IconRegion *ir)
 {
     IconEntry	*ip;
 
@@ -321,9 +308,7 @@ prevIconEntry (ie, ir)
  * regions together
  */
 
-static
-mergeEntries (old, ie)
-    IconEntry	*old, *ie;
+static void mergeEntries (IconEntry *old, IconEntry *ie)
 {
     if (old->y == ie->y) {
 	ie->w = old->w + ie->w;
@@ -336,8 +321,7 @@ mergeEntries (old, ie)
     }
 }
 
-IconDown (tmp_win)
-    TwmWindow   *tmp_win;
+void IconDown (TwmWindow *tmp_win)
 {
     IconEntry	*ie, *ip, *in;
     IconRegion	*ir;
@@ -372,12 +356,10 @@ IconDown (tmp_win)
     }
 }
 
-name_list **
-AddIconRegion(geom, grav1, grav2, stepx, stepy, ijust, just, align)
-char *geom;
-int grav1, grav2;
-int stepx, stepy;
-char *ijust, *just, *align;
+name_list **AddIconRegion(char *geom,
+			  int grav1, int grav2,
+			  int stepx, int stepy,
+			  char *ijust, char *just, char *align)
 {
     IconRegion *ir;
     int mask, tmp;
@@ -403,11 +385,9 @@ char *ijust, *just, *align;
 
     mask = XParseGeometry(geom, &ir->x, &ir->y, (unsigned int *)&ir->w, (unsigned int *)&ir->h);
 
-    if (mask & XNegative)
-	ir->x += Scr->MyDisplayWidth - ir->w;
+    if (mask & XNegative) ir->x += Scr->rootw - ir->w;
+    if (mask & YNegative) ir->y += Scr->rooth - ir->h;
 
-    if (mask & YNegative)
-	ir->y += Scr->MyDisplayHeight - ir->h;
     ir->entries = (IconEntry *)malloc(sizeof(IconEntry));
     ir->entries->next = 0;
     ir->entries->x = ir->x;
@@ -471,9 +451,7 @@ FreeIconRegions()
 }
 #endif
 
-CreateIconWindow(tmp_win, def_x, def_y)
-TwmWindow *tmp_win;
-int def_x, def_y;
+int CreateIconWindow(TwmWindow *tmp_win, int def_x, int def_y)
 {
     unsigned long event_mask;
     unsigned long valuemask;		/* mask for create windows */
@@ -726,7 +704,7 @@ int def_x, def_y;
 	event_mask = ExposureMask;
     }
 
-    if (Scr->AutoRaiseIcons || Scr->SchrinkIconTitles)
+    if (Scr->AutoRaiseIcons || Scr->ShrinkIconTitles)
 	event_mask |= EnterWindowMask | LeaveWindowMask;
     XSelectInput (dpy, icon->w,
 		  KeyPressMask | ButtonPressMask | ButtonReleaseMask | event_mask);
@@ -756,18 +734,18 @@ int def_x, def_y;
 			0, 0, &rect, 1, ShapeSet, 0);
 	}
 	if (icon->has_title) {
-	    if (Scr->SchrinkIconTitles) {
+	    if (Scr->ShrinkIconTitles) {
 		rect.x      = x;
 		rect.y      = icon->height;
 		rect.width  = icon->width;
 		rect.height = icon->w_height - icon->height;
-		icon->title_schrinked = True;
+		icon->title_shrunk = True;
 	    } else {
 		rect.x      = 0;
 		rect.y      = icon->height;
 		rect.width  = icon->w_width;
 		rect.height = icon->w_height - icon->height;
-		icon->title_schrinked = False;
+		icon->title_shrunk = False;
 	    }
 	    XShapeCombineRectangles (dpy, icon->w, ShapeBounding,
 			0, 0, &rect, 1, ShapeUnion, 0);
@@ -788,22 +766,21 @@ int def_x, def_y;
     }
     else
     {
-      if (OCCUPY (tmp_win, Scr->workSpaceMgr.activeWSPC))
+      if (visible (tmp_win))
 	PlaceIcon(tmp_win, def_x, def_y, &final_x, &final_y);
     }
 
-  if (OCCUPY (tmp_win, Scr->workSpaceMgr.activeWSPC) ||
+  if (visible (tmp_win) ||
       (tmp_win->wmhints && tmp_win->wmhints->flags & IconPositionHint)) {
-    if (final_x > Scr->MyDisplayWidth)
-	final_x = Scr->MyDisplayWidth - icon->w_width -
-	    (2 * Scr->IconBorderWidth);
-    if (Scr->SchrinkIconTitles && icon->bm_w) {
+    if (final_x > Scr->rootw)
+	final_x = Scr->rootw - icon->w_width - (2 * Scr->IconBorderWidth);
+    if (Scr->ShrinkIconTitles && icon->bm_w) {
 	if (final_x + (icon->w_width - icon->width) < 0) final_x = 0;
     } else {
 	if (final_x < 0) final_x = 0;
     }
-    if (final_y > Scr->MyDisplayHeight)
-	final_y = Scr->MyDisplayHeight - icon->height -
+    if (final_y > Scr->rooth)
+	final_y = Scr->rooth - icon->height -
 	    Scr->IconFont.height - 6 - (2 * Scr->IconBorderWidth);
     if (final_y < 0) final_y = 0;
 
@@ -819,8 +796,7 @@ int def_x, def_y;
     return (0);
 }
 
-void SchrinkIconTitle (tmp_win)
-TwmWindow *tmp_win;
+void ShrinkIconTitle (TwmWindow *tmp_win)
 {
     Icon	*icon;
     XRectangle	rect;
@@ -836,13 +812,12 @@ TwmWindow *tmp_win;
     rect.width  = icon->width;
     rect.height = icon->w_height;
     XShapeCombineRectangles (dpy, icon->w, ShapeBounding, 0, 0, &rect, 1, ShapeIntersect, 0);
-    icon->title_schrinked = True;
+    icon->title_shrunk = True;
     XClearArea (dpy, icon->w, 0, icon->height, icon->w_width,
 		icon->w_height - icon->height, True);
 }
 
-void ExpandIconTitle (tmp_win)
-TwmWindow *tmp_win;
+void ExpandIconTitle (TwmWindow *tmp_win)
 {
     Icon	*icon;
     XRectangle	rect;
@@ -858,13 +833,12 @@ TwmWindow *tmp_win;
     rect.width  = icon->w_width;
     rect.height = icon->w_height - icon->height;
     XShapeCombineRectangles (dpy, icon->w, ShapeBounding, 0, 0, &rect, 1, ShapeUnion, 0);
-    icon->title_schrinked = False;
+    icon->title_shrunk = False;
     XClearArea (dpy, icon->w, 0, icon->height, icon->w_width,
 		icon->w_height - icon->height, True);
 }
 
-void ReshapeIcon (icon)
-Icon *icon;
+void ReshapeIcon (Icon *icon)
 {
     int x;
     XRectangle	rect;
@@ -889,8 +863,7 @@ Icon *icon;
     XShapeCombineRectangles (dpy, icon->w, ShapeBounding, 0, 0, &rect, 1, ShapeUnion, 0);
 }
 
-int GetIconOffset (icon)
-Icon *icon;
+int GetIconOffset (Icon *icon)
 {
     short justif;
 
@@ -912,9 +885,7 @@ Icon *icon;
     }
 }
 
-Bool AnimateIcons (scr, icon)
-ScreenInfo *scr;
-Icon	   *icon;
+Bool AnimateIcons (ScreenInfo *scr, Icon *icon)
 {
     Image	*image;
     XRectangle	rect;
