@@ -68,6 +68,8 @@
  ***********************************************************************/
 
 #include <stdio.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #if defined(sony_news) || defined __QNX__
 #  include <ctype.h>
 #endif
@@ -75,9 +77,11 @@
 #include <ctype.h>
 #include <decw$include/Xos.h>
 #include <X11Xmu/CharSet.h>
+#include <X11Xmu/SysUtil.h>
 #else
 #include <X11/Xos.h>
 #include <X11/Xmu/CharSet.h>
+#include <X11/Xmu/SysUtil.h>
 #endif
 #include "twm.h"
 #include "screen.h"
@@ -86,6 +90,9 @@
 #include "gram.h"
 #include "parse.h"
 #include "version.h"
+#ifdef SOUNDS
+#  include "sound.h"
+#endif
 #ifdef VMS
 #include <decw$include/Xatom.h> 
 #else
@@ -113,9 +120,10 @@
 #endif
 #define BUF_LEN 300
 
-static int ParseRandomPlacement ();
-static int ParseButtonStyle ();
-extern int yyparse(),twmrc_error_prefix();
+static int ParseRandomPlacement (register char *s);
+static int ParseButtonStyle (register char *s);
+extern int yyparse(void);
+extern void twmrc_error_prefix(void);
 
 static FILE *twmrc;
 static int ptr = 0;
@@ -124,10 +132,10 @@ static char buff[BUF_LEN+1];
 static char overflowbuff[20];		/* really only need one */
 static int overflowlen;
 static char **stringListSource, *currentString;
-static int ParseUsePPosition();
+static int ParseUsePPosition (register char *s);
 #ifdef USEM4
-static FILE *start_m4();
-static char *m4_defs();
+static FILE *start_m4(FILE *fraw);
+static char *m4_defs(Display *display, char *host);
 #endif
 
 extern int twmrc_lineno;
@@ -137,22 +145,20 @@ int ConstrainedMoveTime = 400;		/* milliseconds, event times */
 
 int RaiseDelay = 0;			/* msec, for AutoRaise */
 
-static int twmFileInput(), twmStringListInput();
-#ifdef USEM4
-static int  m4twmFileInput ();
+static int twmStringListInput(void);
+#ifndef USEM4
+static int twmFileInput(void);
+#else
+static int m4twmFileInput (void);
 #endif
-void twmUnput();
-int (*twmInputFunc)();
+void twmUnput(int c);
+int (*twmInputFunc)(void);
 
 extern char *defTwmrc[];		/* default bindings */
 
 extern int captive;
 extern char *captivename;
 
-#ifdef SOUNDS
-extern int set_sound_host();
-#endif
-
 /***********************************************************************
  *
  *  Procedure:
@@ -164,10 +170,11 @@ extern int set_sound_host();
  ***********************************************************************
  */
 
-static int doparse (ifunc, srctypename, srcname)
-    int (*ifunc)();
-    char *srctypename;
-    char *srcname;
+#ifdef YYDEBUG
+int yydebug = 1;
+#endif
+
+static int doparse (int (*ifunc)(void), char *srctypename, char *srcname)
 {
     mods = 0;
     ptr = 0;
@@ -189,8 +196,7 @@ static int doparse (ifunc, srctypename, srcname)
 }
 
 
-int ParseTwmrc (filename)
-    char *filename;
+int ParseTwmrc (char *filename)
 {
     int i;
     char *home = NULL;
@@ -336,6 +342,7 @@ int ParseTwmrc (filename)
 #else
     if (twmrc) {
 #endif
+
 	int status;
 
 	if (filename && strncmp (cp, filename, strlen (filename))) {
@@ -364,8 +371,7 @@ int ParseTwmrc (filename)
     }
 }
 
-int ParseStringList (sl)
-    char **sl;
+int ParseStringList (char **sl)
 {
     stringListSource = sl;
     currentString = *sl;
@@ -456,7 +462,7 @@ static int twmFileInput()
 /* If you're going to use m4, use this version instead.  Much simpler.
  * m4 ism's credit to Josh Osborne (stripes) */
 
-static int m4twmFileInput()
+static int m4twmFileInput(void)
 {
     int line;
     extern char *keepM4_filename;
@@ -499,7 +505,7 @@ static int m4twmFileInput()
 #endif /* USEM4 */
 
 
-static int twmStringListInput()
+static int twmStringListInput(void)
 {
     if (overflowlen) return (int) overflowbuff[--overflowlen];
 
@@ -528,8 +534,7 @@ static int twmStringListInput()
  ***********************************************************************
  */
 
-void twmUnput (c)
-    int c;
+void twmUnput (int c)
 {
     if (overflowlen < sizeof overflowbuff) {
 	overflowbuff[overflowlen++] = (char) c;
@@ -552,8 +557,7 @@ void twmUnput (c)
  ***********************************************************************
  */
 
-void
-TwmOutput(c)
+void TwmOutput(int c)
 {
     putchar(c);
 }
@@ -562,7 +566,7 @@ TwmOutput(c)
 /**********************************************************************
  *
  *  Parsing table and routines
- * 
+ *
  ***********************************************************************/
 
 typedef struct _TwmKeyword {
@@ -606,7 +610,6 @@ typedef struct _TwmKeyword {
 #define kw0_Use3DBorders		36
 #define kw0_SunkFocusWindowTitle	37
 #define kw0_BeNiceToColormap		38
-#define kw0_SmartIconify		39
 #define kw0_WarpRingOnScreen		40
 #define kw0_NoIconManagerFocus		41
 #define kw0_StayUpMenus			42
@@ -619,7 +622,7 @@ typedef struct _TwmKeyword {
 #define kw0_DontWarpCursorInWMap	49
 #define kw0_CenterFeedbackWindow	50
 #define kw0_WarpToDefaultMenuEntry	51
-#define kw0_SchrinkIconTitles		52
+#define kw0_ShrinkIconTitles		52
 #define kw0_AutoRaiseIcons		53
 #define kw0_use3DIconBorders		54
 #define kw0_UseSunkTitlePixmap          55
@@ -631,6 +634,8 @@ typedef struct _TwmKeyword {
 #define kw0_PackNewWindows		61
 #define kw0_IgnoreCaseInMenuSelection	62
 #define kw0_SloppyFocus                 63
+#define kw0_NoImagesInWorkSpaceManager  64
+#define kw0_NoWarpToMenuTitle 65
 
 #define kws_UsePPosition		1
 #define kws_IconFont			2
@@ -652,6 +657,7 @@ typedef struct _TwmKeyword {
 #endif
 #define kws_WMgrButtonStyle		17
 #define kws_WorkSpaceFont               18
+#define kws_IconifyStyle                19
 
 #define kwn_ConstrainedMoveTime		1
 #define kwn_MoveDelta			2
@@ -724,6 +730,7 @@ static TwmKeyword keytable[] = {
     { "alter",			ALTER, 0 },
     { "alwaysontop",		ALWAYS_ON_TOP, 0 },
     { "alwaysshowwindowwhenmovingfromworkspacemanager", KEYWORD, kw0_ShowWinWhenMovingInWmgr },
+    { "alwayssqueezetogravity",	ALWAYSSQUEEZETOGRAVITY, 0 },
     { "animationspeed",		NKEYWORD, kwn_AnimationSpeed },
     { "autofocustotransients",  KEYWORD, kw0_AutoFocusToTransients }, /* kai */
     { "autolower",		AUTO_LOWER, 0 },
@@ -769,6 +776,7 @@ static TwmKeyword keytable[] = {
     { "dontsave",		DONT_SAVE, 0 },
     { "dontsetinactive",	DONTSETINACTIVE, 0 },
     { "dontsqueezetitle",	DONT_SQUEEZE_TITLE, 0 },
+    { "donttoggleworkspacemanagerstate", DONTTOGGLEWORKSPACEMANAGERSTATE, 0 },
     { "dontwarpcursorinwmap",	KEYWORD, kw0_DontWarpCursorInWMap },
     { "east",			DKEYWORD, D_EAST },
     { "f",			FRAME, 0 },
@@ -782,6 +790,7 @@ static TwmKeyword keytable[] = {
     { "f.backmapiconmgr",	FKEYWORD, F_BACKMAPICONMGR },
     { "f.beep",			FKEYWORD, F_BEEP },
     { "f.bottomzoom",		FKEYWORD, F_BOTTOMZOOM },
+    { "f.changesize",           FSKEYWORD, F_CHANGESIZE },
     { "f.circledown",		FKEYWORD, F_CIRCLEDOWN },
     { "f.circleup",		FKEYWORD, F_CIRCLEUP },
     { "f.colormap",		FSKEYWORD, F_COLORMAP },
@@ -829,6 +838,10 @@ static TwmKeyword keytable[] = {
     { "f.movepack",		FKEYWORD, F_MOVEPACK },
     { "f.movepush",		FKEYWORD, F_MOVEPUSH },
     { "f.moveresize",		FSKEYWORD, F_MOVERESIZE },
+    { "f.movetonextworkspace",  FKEYWORD, F_MOVETONEXTWORKSPACE },
+    { "f.movetonextworkspaceandfollow",  FKEYWORD, F_MOVETONEXTWORKSPACEANDFOLLOW },
+    { "f.movetoprevworkspace",  FKEYWORD, F_MOVETOPREVWORKSPACE },
+    { "f.movetoprevworkspaceandfollow",  FKEYWORD, F_MOVETOPREVWORKSPACEANDFOLLOW },
     { "f.nexticonmgr",		FKEYWORD, F_NEXTICONMGR },
     { "f.nextworkspace",	FKEYWORD, F_NEXTWORKSPACE },
     { "f.nop",			FKEYWORD, F_NOP },
@@ -859,6 +872,7 @@ static TwmKeyword keytable[] = {
     { "f.separator",		FKEYWORD, F_SEPARATOR },
     { "f.setbuttonsstate",	FKEYWORD, F_SETBUTTONSTATE },
     { "f.setmapstate",		FKEYWORD, F_SETMAPSTATE },
+    { "f.showbackground",      	FKEYWORD, F_SHOWBGRD },
     { "f.showiconmgr",		FKEYWORD, F_SHOWLIST },
     { "f.showworkspacemgr",	FKEYWORD, F_SHOWWORKMGR },
     { "f.slowdownanimation",	FKEYWORD, F_SLOWDOWNANIMATION },
@@ -906,6 +920,7 @@ static TwmKeyword keytable[] = {
     { "iconforeground",		CLKEYWORD, kwcl_IconForeground },
     { "iconifybyunmapping",	ICONIFY_BY_UNMAPPING, 0 },
     { "iconifyfunction",	ICONIFY_FUNCTION, 0 },
+    { "iconifystyle",		SKEYWORD, kws_IconifyStyle },
     { "iconjustification",	SKEYWORD, kws_IconJustification },
     { "iconmanagerbackground",	CLKEYWORD, kwcl_IconManagerBackground },
     { "iconmanagerdontshow",	ICONMGR_NOSHOW, 0 },
@@ -925,6 +940,7 @@ static TwmKeyword keytable[] = {
     { "ignorecaseinmenuselection",	KEYWORD, kw0_IgnoreCaseInMenuSelection },
     { "ignorelockmodifier",	KEYWORD, kw0_IgnoreLockModifier },
     { "ignoremodifier",		IGNOREMODIFIER, 0 },
+    { "ignoretransient",	IGNORE_TRANSIENT, 0 },
     { "interpolatemenucolors",	KEYWORD, kw0_InterpolateMenuColors },
     { "l",			LOCK, 0 },
     { "left",			JKEYWORD, J_LEFT },
@@ -962,6 +978,7 @@ static TwmKeyword keytable[] = {
     { "noiconmanagerfocus",	KEYWORD, kw0_NoIconManagerFocus },
     { "noiconmanagers",		KEYWORD, kw0_NoIconManagers },
     { "noicontitle",		NO_ICON_TITLE, 0  },
+    { "noimagesinworkspacemanager", KEYWORD, kw0_NoImagesInWorkSpaceManager },
     { "nomenushadows",		KEYWORD, kw0_NoMenuShadows },
     { "noopaquemove",		NOOPAQUEMOVE, 0 },
     { "noopaqueresize",		NOOPAQUERESIZE, 0 },
@@ -977,6 +994,7 @@ static TwmKeyword keytable[] = {
     { "notitlefocus",		KEYWORD, kw0_NoTitleFocus },
     { "notitlehighlight",	NO_TITLE_HILITE, 0 },
     { "noversion",		KEYWORD, kw0_NoVersion },
+    { "nowarptomenutitle",      KEYWORD, kw0_NoWarpToMenuTitle },
     { "occupy",			OCCUPYLIST, 0 },
     { "occupyall",		OCCUPYALL, 0 },
     { "opaquemove",		OPAQUEMOVE, 0 },
@@ -1003,14 +1021,14 @@ static TwmKeyword keytable[] = {
     { "root",			ROOT, 0 },
     { "s",			SHIFT, 0 },
     { "savecolor",              SAVECOLOR, 0},
-    { "schrinkicontitles",	KEYWORD, kw0_SchrinkIconTitles },
+    { "schrinkicontitles",	KEYWORD, kw0_ShrinkIconTitles },
+    { "shrinkicontitles",	KEYWORD, kw0_ShrinkIconTitles },
     { "select",			SELECT, 0 },
     { "shift",			SHIFT, 0 },
     { "shortallwindowsmenus",	KEYWORD, kw0_ShortAllWindowsMenus },
     { "showiconmanager",	KEYWORD, kw0_ShowIconManager },
     { "showworkspacemanager",	KEYWORD, kw0_ShowWorkspaceManager },
     { "sloppyfocus",            KEYWORD, kw0_SloppyFocus },
-    { "smarticonify",		KEYWORD, kw0_SmartIconify },
     { "sorticonmanager",	KEYWORD, kw0_SortIconManager },
 #ifdef SOUNDS
     { "soundhost",		SKEYWORD, kws_SoundHost },
@@ -1046,6 +1064,7 @@ static TwmKeyword keytable[] = {
     { "usethreedmenus",		KEYWORD, kw0_Use3DMenus },
     { "usethreedtitles",	KEYWORD, kw0_Use3DTitles },
     { "usethreedwmap",		KEYWORD, kw0_Use3DWMap },
+    { "virtualscreens",         VIRTUAL_SCREENS, 0 },
     { "w",			WINDOW, 0 },
     { "wait",			WAITC, 0 },
     { "warpcursor",		WARP_CURSOR, 0 },
@@ -1077,9 +1096,7 @@ static TwmKeyword keytable[] = {
 
 static int numkeywords = (sizeof(keytable)/sizeof(keytable[0]));
 
-int parse_keyword (s, nump)
-    char *s;
-    int *nump;
+int parse_keyword (char *s, int *nump)
 {
     register int lower = 0, upper = numkeywords - 1;
 
@@ -1107,9 +1124,10 @@ int parse_keyword (s, nump)
  * action routines called by grammar
  */
 
-int do_single_keyword (keyword)
-    int keyword;
+int do_single_keyword (int keyword)
 {
+    virtualScreen *vs;
+
     switch (keyword) {
       case kw0_NoDefaults:
 	Scr->NoDefaults = TRUE;
@@ -1196,11 +1214,14 @@ int do_single_keyword (keyword)
 	return 1;
 
       case kw0_StartInMapState:
-	Scr->workSpaceMgr.workspaceWindow.state = MAPSTATE;
+	for (vs = Scr->vScreenList; vs != NULL; vs = vs->next) {
+	  vs->wsw->state = MAPSTATE;
+	}
+	Scr->workSpaceMgr.initialstate = MAPSTATE; /* redundant, really */
 	return 1;
 
       case kw0_NoShowOccupyAll:
-	Scr->workSpaceMgr.workspaceWindow.noshowoccupyall = TRUE;
+	Scr->workSpaceMgr.noshowoccupyall = TRUE;
 	return 1;
 
       case kw0_AutoOccupy:
@@ -1247,10 +1268,6 @@ int do_single_keyword (keyword)
 	Scr->BeNiceToColormap = TRUE;
 	return 1;
 
-      case kw0_SmartIconify:
-	Scr->SmartIconify = TRUE;
-	return 1;
-
       case kw0_BorderResizeCursors:
 	Scr->BorderCursors = TRUE;
 	return 1;
@@ -1307,8 +1324,8 @@ int do_single_keyword (keyword)
 	Scr->WarpToDefaultMenuEntry = TRUE;
 	return 1;
 
-      case kw0_SchrinkIconTitles:
-	Scr->SchrinkIconTitles = TRUE;
+      case kw0_ShrinkIconTitles:
+	Scr->ShrinkIconTitles = TRUE;
 	return 1;
 
       case kw0_AutoRaiseIcons:
@@ -1352,30 +1369,36 @@ int do_single_keyword (keyword)
 	Scr->SloppyFocus = TRUE;
 	return 1;
 
+      case kw0_NoImagesInWorkSpaceManager:
+	Scr->NoImagesInWorkSpaceManager = TRUE;
+	return 1;
+
+ case kw0_NoWarpToMenuTitle:
+ Scr->NoWarpToMenuTitle = TRUE;
+ return 1;
+
     }
     return 0;
 }
 
 
-int do_string_keyword (keyword, s)
-    int keyword;
-    char *s;
+int do_string_keyword (int keyword, char *s)
 {
     switch (keyword) {
        case kws_RandomPlacement:
- 	{
- 	    int rp = ParseRandomPlacement (s);
- 	    if (rp < 0) {
- 		twmrc_error_prefix();
- 		fprintf (stderr,
- 			 "ignoring invalid RandomPlacement argument \"%s\"\n", s);
- 	    } else {
- 		Scr->RandomPlacement = rp;
- 	    }
- 	    return 1;
- 	}
+	{
+	    int rp = ParseRandomPlacement (s);
+	    if (rp < 0) {
+		twmrc_error_prefix();
+		fprintf (stderr,
+			 "ignoring invalid RandomPlacement argument \"%s\"\n", s);
+	    } else {
+		Scr->RandomPlacement = rp;
+	    }
+	    return 1;
+	}
       case kws_UsePPosition:
-	{ 
+	{
 	    int ppos = ParseUsePPosition (s);
 	    if (ppos < 0) {
 		twmrc_error_prefix();
@@ -1401,7 +1424,7 @@ int do_string_keyword (keyword, s)
 	return 1;
 
       case kws_WorkSpaceFont:
-	if (!Scr->HaveFonts) Scr->workSpaceMgr.workspaceWindow.windowFont.basename = s;
+	if (!Scr->HaveFonts) Scr->workSpaceMgr.windowFont.basename = s;
 	return 1;
 
       case kws_TitleFont:
@@ -1425,7 +1448,7 @@ int do_string_keyword (keyword, s)
 	return 1;
 
       case kws_WorkSpaceFont:
-	if (!Scr->HaveFonts) Scr->workSpaceMgr.workspaceWindow.windowFont.name = s;
+	if (!Scr->HaveFonts) Scr->workSpaceMgr.windowFont.name = s;
 	return 1;
 
       case kws_TitleFont:
@@ -1457,9 +1480,9 @@ int do_string_keyword (keyword, s)
 	    fprintf (stderr, "bad MaxWindowSize \"%s\"\n", s);
 	    return 0;
 	}
-	if (JunkWidth <= 0 || JunkHeight <= 0) {
+	if (JunkWidth == 0 || JunkHeight == 0) {
 	    twmrc_error_prefix();
-	    fprintf (stderr, "MaxWindowSize \"%s\" must be positive\n", s);
+	    fprintf (stderr, "MaxWindowSize \"%s\" must be non-zero\n", s);
 	    return 0;
 	}
 	Scr->MaxWindowWidth = JunkWidth;
@@ -1534,19 +1557,35 @@ int do_string_keyword (keyword, s)
  		fprintf (stderr,
  			 "ignoring invalid WMgrButtonStyle argument \"%s\"\n", s);
  	    } else {
- 		Scr->workSpaceMgr.workspaceWindow.buttonStyle = style;
+ 		Scr->workSpaceMgr.buttonStyle = style;
  	    }
  	    return 1;
 	}
+
+      case kws_IconifyStyle:
+	{
+	  if (strlen (s) == 0) {
+	    twmrc_error_prefix();
+	    fprintf (stderr, "ignoring invalid IconifyStyle argument \"%s\"\n", s);
+	  }
+	  if (strcmp (s,  "default") == 0) Scr->IconifyStyle = ICONIFY_NORMAL;
+	  XmuCopyISOLatin1Lowered (s, s);
+	  if (strcmp (s,  "normal") == 0) Scr->IconifyStyle = ICONIFY_NORMAL;
+	  if (strcmp (s,  "mosaic") == 0) Scr->IconifyStyle = ICONIFY_MOSAIC;
+	  if (strcmp (s,  "zoomin") == 0) Scr->IconifyStyle = ICONIFY_ZOOMIN;
+	  if (strcmp (s, "zoomout") == 0) Scr->IconifyStyle = ICONIFY_ZOOMOUT;
+	  if (strcmp (s,   "sweep") == 0) Scr->IconifyStyle = ICONIFY_SWEEP;
+	  return 1;
+	}
     }
     return 0;
 }
 
 
-int do_number_keyword (keyword, num)
-    int keyword;
-    int num;
+int do_number_keyword (int keyword, int num)
 {
+    virtualScreen *vs;
+
     switch (keyword) {
       case kwn_ConstrainedMoveTime:
 	ConstrainedMoveTime = num;
@@ -1630,11 +1669,19 @@ int do_number_keyword (keyword, num)
       case kwn_WMgrVertButtonIndent:
 	if (Scr->FirstTime) Scr->WMgrVertButtonIndent = num;
 	if (Scr->WMgrVertButtonIndent < 0) Scr->WMgrVertButtonIndent = 0;
+	for (vs = Scr->vScreenList; vs != NULL; vs = vs->next) {
+	  vs->wsw->vspace          = Scr->WMgrVertButtonIndent;
+	}
+	Scr->workSpaceMgr.occupyWindow->vspace = Scr->WMgrVertButtonIndent;
 	return 1;
 
       case kwn_WMgrHorizButtonIndent:
 	if (Scr->FirstTime) Scr->WMgrHorizButtonIndent = num;
 	if (Scr->WMgrHorizButtonIndent < 0) Scr->WMgrHorizButtonIndent = 0;
+	for (vs = Scr->vScreenList; vs != NULL; vs = vs->next) {
+	  vs->wsw->hspace          = Scr->WMgrHorizButtonIndent;
+	}
+	Scr->workSpaceMgr.occupyWindow->hspace = Scr->WMgrHorizButtonIndent;
 	return 1;
 
       case kwn_WMgrButtonShadowDepth:
@@ -1724,10 +1771,7 @@ int do_number_keyword (keyword, num)
     return 0;
 }
 
-name_list **do_colorlist_keyword (keyword, colormode, s)
-    int keyword;
-    int colormode;
-    char *s;
+name_list **do_colorlist_keyword (int keyword, int colormode, char *s)
 {
     switch (keyword) {
       case kwcl_BorderColor:
@@ -1775,22 +1819,19 @@ name_list **do_colorlist_keyword (keyword, colormode, s)
 	return &Scr->IconManagerBL;
 
       case kwcl_MapWindowBackground:
-	GetColor (colormode, &Scr->workSpaceMgr.workspaceWindow.windowcp.back, s);
-	Scr->workSpaceMgr.workspaceWindow.windowcpgiven = True;
-	return &Scr->workSpaceMgr.workspaceWindow.windowBackgroundL;
+	GetColor (colormode, &Scr->workSpaceMgr.windowcp.back, s);
+	Scr->workSpaceMgr.windowcpgiven = True;
+	return &Scr->workSpaceMgr.windowBackgroundL;
 
       case kwcl_MapWindowForeground:
-	GetColor (colormode, &Scr->workSpaceMgr.workspaceWindow.windowcp.fore, s);
-	Scr->workSpaceMgr.workspaceWindow.windowcpgiven = True;
-	return &Scr->workSpaceMgr.workspaceWindow.windowForegroundL;
+	GetColor (colormode, &Scr->workSpaceMgr.windowcp.fore, s);
+	Scr->workSpaceMgr.windowcpgiven = True;
+	return &Scr->workSpaceMgr.windowForegroundL;
     }
     return NULL;
 }
 
-int do_color_keyword (keyword, colormode, s)
-    int keyword;
-    int colormode;
-    char *s;
+int do_color_keyword (int keyword, int colormode, char *s)
 {
     switch (keyword) {
       case kwc_DefaultForeground:
@@ -1829,35 +1870,32 @@ int do_color_keyword (keyword, colormode, s)
 /*
  * put_pixel_on_root() Save a pixel value in twm root window color property.
  */
-put_pixel_on_root(pixel)                                 
-    Pixel pixel;                                         
-{                                                        
+static void put_pixel_on_root(Pixel pixel)
+{
   int           i, addPixel = 1;
-  Atom          pixelAtom, retAtom;	                 
+  Atom          pixelAtom, retAtom;
   int           retFormat;
-  unsigned long nPixels, retAfter;                     
+  unsigned long nPixels, retAfter;
   Pixel        *retProp;
-  pixelAtom = XInternAtom(dpy, "_MIT_PRIORITY_COLORS", True);        
-  XGetWindowProperty(dpy, Scr->Root, pixelAtom, 0, 8192, 
-		     False, XA_CARDINAL, &retAtom,       
-		     &retFormat, &nPixels, &retAfter,    
+  pixelAtom = XInternAtom(dpy, "_MIT_PRIORITY_COLORS", True);
+  XGetWindowProperty(dpy, Scr->Root, pixelAtom, 0, 8192,
+		     False, XA_CARDINAL, &retAtom,
+		     &retFormat, &nPixels, &retAfter,
 		     (unsigned char **)&retProp);
 
-  for (i=0; i< nPixels; i++)                             
-      if (pixel == retProp[i]) addPixel = 0;             
-                                                         
-  if (addPixel)                                          
+  for (i=0; i< nPixels; i++)
+      if (pixel == retProp[i]) addPixel = 0;
+
+  if (addPixel)
       XChangeProperty (dpy, Scr->Root, _XA_MIT_PRIORITY_COLORS,
-		       XA_CARDINAL, 32, PropModeAppend,  
-		       (unsigned char *)&pixel, 1);                       
-}                                                        
+		       XA_CARDINAL, 32, PropModeAppend,
+		       (unsigned char *)&pixel, 1);
+}
 
 /*
  * do_string_savecolor() save a color from a string in the twmrc file.
  */
-int do_string_savecolor(colormode, s)
-     int colormode;
-     char *s;
+int do_string_savecolor(int colormode, char *s)
 {
   Pixel p;
   GetColor(colormode, &p, s);
@@ -1871,8 +1909,7 @@ int do_string_savecolor(colormode, s)
 typedef struct _cnode {int i; struct _cnode *next;} Cnode, *Cptr;
 Cptr chead = NULL;
 
-int do_var_savecolor(key)
-int key;
+int do_var_savecolor(int key)
 {
   Cptr cptrav, cpnew;
   if (!chead) {
@@ -1892,7 +1929,7 @@ int key;
  * assign_var_savecolor() traverse the var save color list placeing the pixels
  *                        in the root window property.
  */
-void assign_var_savecolor()
+void assign_var_savecolor(void)
 {
   Cptr cp = chead;
   while (cp != NULL) {
@@ -1931,10 +1968,10 @@ void assign_var_savecolor()
       put_pixel_on_root(Scr->IconManagerC.back);
       break;
     case kwcl_MapWindowForeground:
-      put_pixel_on_root(Scr->workSpaceMgr.workspaceWindow.windowcp.fore);
+      put_pixel_on_root(Scr->workSpaceMgr.windowcp.fore);
       break;
     case kwcl_MapWindowBackground:
-      put_pixel_on_root(Scr->workSpaceMgr.workspaceWindow.windowcp.back);
+      put_pixel_on_root(Scr->workSpaceMgr.windowcp.back);
       break;
     }
     cp = cp->next;
@@ -1945,8 +1982,7 @@ void assign_var_savecolor()
   }
 }
 
-static int ParseRandomPlacement (s)
-    register char *s;
+static int ParseRandomPlacement (register char *s)
 {
     if (strlen (s) == 0) return RP_ALL;
     if (strcmp (s, "default") == 0) return RP_ALL;
@@ -1959,8 +1995,7 @@ static int ParseRandomPlacement (s)
     return (-1);
 }
 
-int ParseJustification (s)
-    register char *s;
+int ParseJustification (register char *s)
 {
     if (strlen (s) == 0) return (-1);
     if (strcmp (s, "default") == 0) return J_CENTER;
@@ -1976,8 +2011,7 @@ int ParseJustification (s)
     return (-1);
 }
 
-int ParseAlignement (s)
-    register char *s;
+int ParseAlignement (register char *s)
 {
     if (strlen (s) == 0) return (-1);
     if (strcmp (s, "default") == 0) return J_CENTER;
@@ -1993,8 +2027,7 @@ int ParseAlignement (s)
     return (-1);
 }
 
-static int ParseUsePPosition (s)
-    register char *s;
+static int ParseUsePPosition (register char *s)
 {
     if (strlen (s) == 0) return (-1);
     if (strcmp (s,  "default") == 0) return PPOS_OFF;
@@ -2008,8 +2041,7 @@ static int ParseUsePPosition (s)
     return (-1);
 }
 
-static int ParseButtonStyle (s)
-    register char *s;
+static int ParseButtonStyle (register char *s)
 {
     if (strlen (s) == 0) return (-1);
     if (strcmp (s,  "default") == 0) return STYLE_NORMAL;
@@ -2022,12 +2054,11 @@ static int ParseButtonStyle (s)
     return (-1);
 }
 
-do_squeeze_entry (list, name, justify, num, denom)
-    name_list **list;			/* squeeze or dont-squeeze list */
-    char *name;				/* window name */
-    int justify;			/* left, center, right */
-    int num;				/* signed num */
-    int denom;				/* 0 or indicates fraction denom */
+int do_squeeze_entry (name_list **list,	/* squeeze or dont-squeeze list */
+		      char *name,	/* window name */
+		      int justify,	/* left, center, right */
+		      int num,		/* signed num */
+		      int denom)	/* 0 or indicates fraction denom */
 {
     int absnum = (num < 0 ? -num : num);
 
@@ -2070,8 +2101,7 @@ do_squeeze_entry (list, name, justify, num, denom)
 
 #ifdef USEM4
 
-static FILE *start_m4(fraw)
-FILE *fraw;
+static FILE *start_m4(FILE *fraw)
 {
         int fno;
         int fids[2];
@@ -2112,8 +2142,7 @@ FILE *fraw;
 #define Resolution(pixels, mm)  ((((pixels) * 100000 / (mm)) + 50) / 100)
 #define EXTRA   16
 
-static char *MkDef(name, def)
-char *name, *def;
+static char *MkDef(char *name, char *def)
 {
         static char *cp = NULL;
         static int maxsize = 0;
@@ -2141,9 +2170,7 @@ char *name, *def;
         return(cp);
 }
 
-static char *MkNum(name, def)
-char *name;
-int def;
+static char *MkNum(char *name, int def)
 {
         char num[20];
 
@@ -2164,9 +2191,7 @@ char *str;
 }
 #endif
 
-static char *m4_defs(display, host)
-Display *display;
-char *host;
+static char *m4_defs(Display *display, char *host)
 {
         extern int KeepTmpFile;
         Screen *screen;
@@ -2206,6 +2231,7 @@ char *host;
 	if (!(user=getenv("USER")) && !(user=getenv("LOGNAME"))) user = "unknown";
         fputs(MkDef("USER", user), tmpf);
         fputs(MkDef("HOME", getenv("HOME")), tmpf);
+        fputs(MkDef("PIXMAP_DIRECTORY", PIXMAP_DIRECTORY), tmpf);
         fputs(MkNum("VERSION", ProtocolVersion(display)), tmpf);
         fputs(MkNum("REVISION", ProtocolRevision(display)), tmpf);
         fputs(MkDef("VENDOR", ServerVendor(display)), tmpf);
@@ -2237,6 +2263,24 @@ char *host;
         }
 #ifdef XPM
 	fputs(MkDef("XPM", "Yes"), tmpf);
+#endif
+#ifdef JPEG
+	fputs(MkDef("JPEG", "Yes"), tmpf);
+#endif
+#ifdef IMCONV
+	fputs(MkDef("IMCONV", "Yes"), tmpf);
+#endif
+#ifdef GNOME
+	fputs(MkDef("GNOME", "Yes"), tmpf);
+#endif
+#ifdef SOUNDS
+	fputs(MkDef("SOUNDS", "Yes"), tmpf);
+#endif
+#ifdef X11R6
+	fputs(MkDef("SESSION", "Yes"), tmpf);
+#endif
+#ifdef I18N
+	fputs(MkDef("I18N", "Yes"), tmpf);
 #endif
 	if (captive && captivename) {
             fputs (MkDef ("TWM_CAPTIVE", "Yes"), tmpf);
