@@ -68,6 +68,7 @@
  ***********************************************************************/
 
 #include <stdio.h>
+#include <ctype.h>
 #include "twm.h"
 #include "parse.h"
 #include "events.h"
@@ -81,8 +82,8 @@
 
 static int dragx;       /* all these variables are used */
 static int dragy;       /* in resize operations */
-static int dragWidth;
-static int dragHeight;
+static unsigned int dragWidth;
+static unsigned int dragHeight;
 
 static int origx;
 static int origy;
@@ -104,9 +105,7 @@ static unsigned int resizeGrabMask;
 extern Cursor	TopRightCursor, TopLeftCursor, BottomRightCursor, BottomLeftCursor,
 		LeftCursor, RightCursor, TopCursor, BottomCursor;
 
-static void do_auto_clamp (tmp_win, evp)
-    TwmWindow *tmp_win;
-    XEvent *evp;
+static void do_auto_clamp (TwmWindow *tmp_win, XEvent *evp)
 {
     Window junkRoot;
     int x, y, h, v, junkbw;
@@ -162,13 +161,10 @@ static void do_auto_clamp (tmp_win, evp)
  ***********************************************************************
  */
 
-void
-StartResize(evp, tmp_win, fromtitlebar, from3dborder)
-XEvent *evp;
-TwmWindow *tmp_win;
-Bool fromtitlebar, from3dborder;
+void StartResize(XEvent *evp, TwmWindow *tmp_win,
+		 Bool fromtitlebar, Bool from3dborder)
 {
-    Window      junkRoot;
+    Window      junkRoot, grabwin;
     unsigned int junkbw, junkDepth;
     Cursor	cursor;
 
@@ -177,9 +173,11 @@ Bool fromtitlebar, from3dborder;
     if (! Scr->OpaqueResize || resizeWhenAdd) XGrabServer(dpy);
     resizeGrabMask = ButtonPressMask | ButtonReleaseMask |
 			ButtonMotionMask | PointerMotionHintMask;
-    XGrabPointer(dpy, Scr->Root, True, resizeGrabMask,
-        GrabModeAsync, GrabModeAsync,
-        tmp_win->winbox ? tmp_win->winbox->window : Scr->Root, cursor, CurrentTime);
+
+    grabwin = Scr->Root;
+    if (tmp_win->winbox) grabwin = tmp_win->winbox->window;
+    XGrabPointer(dpy, grabwin, True, resizeGrabMask,
+        GrabModeAsync, GrabModeAsync, grabwin, cursor, CurrentTime);
 
     XGetGeometry(dpy, (Drawable) tmp_win->frame, &junkRoot,
         &dragx, &dragy, (unsigned int *)&dragWidth, (unsigned int *)&dragHeight, &junkbw,
@@ -214,10 +212,7 @@ Bool fromtitlebar, from3dborder;
 
 
 
-void
-MenuStartResize(tmp_win, x, y, w, h)
-TwmWindow *tmp_win;
-int x, y, w, h;
+void MenuStartResize(TwmWindow *tmp_win, int x, int y, int w, int h)
 {
     if (! Scr->OpaqueResize) XGrabServer(dpy);
     resizeGrabMask = ButtonPressMask | ButtonMotionMask | PointerMotionMask;
@@ -258,10 +253,7 @@ int x, y, w, h;
  ***********************************************************************
  */
 
-void
-AddStartResize(tmp_win, x, y, w, h)
-TwmWindow *tmp_win;
-int x, y, w, h;
+void AddStartResize(TwmWindow *tmp_win, int x, int y, int w, int h)
 {
     XGrabServer(dpy);
     resizeGrabMask = ButtonReleaseMask | ButtonMotionMask | PointerMotionHintMask;
@@ -288,14 +280,10 @@ int x, y, w, h;
 
 
 
-void
-MenuDoResize(x_root, y_root, tmp_win)
-int x_root;
-int y_root;
-TwmWindow *tmp_win;
+void MenuDoResize(int x_root, int y_root, TwmWindow *tmp_win)
 {
     int action;
-    Cursor cursor;
+    Cursor cursor = 0;
 
     action = 0;
 
@@ -304,7 +292,7 @@ TwmWindow *tmp_win;
 
     if (clampTop) {
         int         delta = y_root - dragy;
-        if (dragHeight - delta < MINHEIGHT) {
+        if ((int)(dragHeight - delta) < MINHEIGHT) {
             delta = dragHeight - MINHEIGHT;
             clampTop = 0;
         }
@@ -326,7 +314,7 @@ TwmWindow *tmp_win;
     }
     if (clampLeft) {
         int         delta = x_root - dragx;
-        if (dragWidth - delta < MINWIDTH) {
+        if ((int)(dragWidth - delta) < MINWIDTH) {
             delta = dragWidth - MINWIDTH;
             clampLeft = 0;
         }
@@ -348,7 +336,7 @@ TwmWindow *tmp_win;
     }
     if (clampBottom) {
         int         delta = y_root - dragy - dragHeight;
-        if (dragHeight + delta < MINHEIGHT) {
+        if ((int)(dragHeight + delta) < MINHEIGHT) {
             delta = MINHEIGHT - dragHeight;
             clampBottom = 0;
         }
@@ -367,7 +355,7 @@ TwmWindow *tmp_win;
     }
     if (clampRight) {
         int         delta = x_root - dragx - dragWidth;
-        if (dragWidth + delta < MINWIDTH) {
+        if ((int)(dragWidth + delta) < MINWIDTH) {
             delta = MINWIDTH - dragWidth;
             clampRight = 0;
         }
@@ -426,14 +414,10 @@ TwmWindow *tmp_win;
  ***********************************************************************
  */
 
-void
-DoResize(x_root, y_root, tmp_win)
-int x_root;
-int y_root;
-TwmWindow *tmp_win;
+void DoResize(int x_root, int y_root, TwmWindow *tmp_win)
 {
     int action;
-    Cursor cursor;
+    Cursor cursor = 0;
 
     action = 0;
 
@@ -442,7 +426,7 @@ TwmWindow *tmp_win;
 
     if (clampTop) {
         int         delta = y_root - dragy;
-        if (dragHeight - delta < MINHEIGHT) {
+        if ((int)(dragHeight - delta) < MINHEIGHT) {
             delta = dragHeight - MINHEIGHT;
             clampTop = 0;
         }
@@ -464,7 +448,7 @@ TwmWindow *tmp_win;
     }
     if (clampLeft) {
         int         delta = x_root - dragx;
-        if (dragWidth - delta < MINWIDTH) {
+        if ((int)(dragWidth - delta) < MINWIDTH) {
             delta = dragWidth - MINWIDTH;
             clampLeft = 0;
         }
@@ -486,7 +470,7 @@ TwmWindow *tmp_win;
     }
     if (clampBottom) {
         int         delta = y_root - dragy - dragHeight;
-        if (dragHeight + delta < MINHEIGHT) {
+        if ((int)(dragHeight + delta) < MINHEIGHT) {
             delta = MINHEIGHT - dragHeight;
             clampBottom = 0;
         }
@@ -507,7 +491,7 @@ TwmWindow *tmp_win;
     }
     if (clampRight) {
         int         delta = x_root - dragx - dragWidth;
-        if (dragWidth + delta < MINWIDTH) {
+        if ((int)(dragWidth + delta) < MINWIDTH) {
             delta = MINWIDTH - dragWidth;
             clampRight = 0;
         }
@@ -567,11 +551,7 @@ TwmWindow *tmp_win;
  ***********************************************************************
  */
 
-void
-DisplaySize(tmp_win, width, height)
-TwmWindow *tmp_win;
-int width;
-int height;
+void DisplaySize(TwmWindow *tmp_win, int width, int height)
 {
     char str[100];
     int dwidth;
@@ -637,8 +617,7 @@ int height;
  ***********************************************************************
  */
 
-void
-EndResize()
+void EndResize(void)
 {
     TwmWindow *tmp_win;
 
@@ -680,17 +659,15 @@ EndResize()
     ResizeWindow = None;
 }
 
-void
-MenuEndResize(tmp_win)
-TwmWindow *tmp_win;
+void MenuEndResize(TwmWindow *tmp_win)
 {
     MoveOutline(Scr->Root, 0, 0, 0, 0, 0, 0);
     XUnmapWindow(dpy, Scr->SizeWindow);
     ConstrainSize (tmp_win, &dragWidth, &dragHeight);
-    AddingX = dragx;
-    AddingY = dragy;
-    AddingW = dragWidth + (2 * tmp_win->frame_bw);
-    AddingH = dragHeight + (2 * tmp_win->frame_bw);
+    AddingX = dragx - tmp_win->frame_bw;
+    AddingY = dragy - tmp_win->frame_bw;
+    AddingW = dragWidth;
+    AddingH = dragHeight;
     SetupWindow (tmp_win, AddingX, AddingY, AddingW, AddingH, -1);
 }
 
@@ -704,9 +681,7 @@ TwmWindow *tmp_win;
  ***********************************************************************
  */
 
-void
-AddEndResize(tmp_win)
-TwmWindow *tmp_win;
+void AddEndResize(TwmWindow *tmp_win)
 {
 
 #ifdef DEBUG
@@ -731,9 +706,8 @@ TwmWindow *tmp_win;
  * 
  ***********************************************************************/
 
-ConstrainSize (tmp_win, widthp, heightp)
-    TwmWindow *tmp_win;
-    int *widthp, *heightp;
+void ConstrainSize (TwmWindow *tmp_win,
+		    unsigned int *widthp, unsigned int *heightp)
 {
 #define makemult(a,b) ((b==1) ? (a) : (((int)((a)/(b))) * (b)) )
 #define _min(a,b) (((a) < (b)) ? (a) : (b))
@@ -883,17 +857,13 @@ ConstrainSize (tmp_win, widthp, heightp)
  ***********************************************************************
  */
 
-void SetupWindow (tmp_win, x, y, w, h, bw)
-    TwmWindow *tmp_win;
-    int x, y, w, h, bw;
+void SetupWindow (TwmWindow *tmp_win, int x, int y, int w, int h, int bw)
 {
     SetupFrame (tmp_win, x, y, w, h, bw, False);
 }
 
-void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
-    TwmWindow *tmp_win;
-    int x, y, w, h, bw;
-    Bool sendEvent;			/* whether or not to force a send */
+void SetupFrame (TwmWindow *tmp_win, int x, int y, int w, int h, int bw,
+		 Bool sendEvent)	/* whether or not to force a send */
 {
     XEvent client_event;
     XWindowChanges frame_wc, xwc;
@@ -906,10 +876,10 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
 	     x, y, w, h, bw);
 #endif
 
-    if (x >= Scr->MyDisplayWidth)
-      x = Scr->MyDisplayWidth - 16;	/* one "average" cursor width */
-    if (y >= Scr->MyDisplayHeight)
-      y = Scr->MyDisplayHeight - 16;	/* one "average" cursor width */
+    if (x >= Scr->rootw)
+      x = Scr->rootw - 16;	/* one "average" cursor width */
+    if (y >= Scr->rooth)
+      y = Scr->rooth - 16;	/* one "average" cursor width */
     if (bw < 0)
       bw = tmp_win->frame_bw;		/* -1 means current frame width */
 
@@ -991,9 +961,9 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
     }
     tmp_win->frame_x = x;
     tmp_win->frame_y = y;
-    if (tmp_win->UnmapByMovingFarAway && !VISIBLE(tmp_win)) {
-	frame_wc.x = Scr->MyDisplayWidth  + 1;
-	frame_wc.y = Scr->MyDisplayHeight + 1;
+    if (tmp_win->UnmapByMovingFarAway && !visible(tmp_win)) {
+	frame_wc.x = Scr->rootw  + 1;
+	frame_wc.y = Scr->rooth + 1;
     } else {
 	frame_wc.x = tmp_win->frame_x;
 	frame_wc.y = tmp_win->frame_y;
@@ -1014,7 +984,7 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
     {
 	xwc.width = (tmp_win->name_x - tmp_win->highlightxl - 2);
         if (xwc.width <= 0) {
-            xwc.x = Scr->MyDisplayWidth;	/* move offscreen */
+            xwc.x = Scr->rootw;	/* move offscreen */
             xwc.width = 1;
         } else {
             xwc.x = tmp_win->highlightxl;
@@ -1029,7 +999,7 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
 	if (Scr->TBInfo.nright > 0) xwc.width -= 2 * Scr->TitlePadding;
 	if (Scr->use3Dtitles) xwc.width -= Scr->TitleButtonShadowDepth;
         if (xwc.width <= 0) {
-            xwc.x = Scr->MyDisplayWidth;	/* move offscreen */
+            xwc.x = Scr->rootw;	/* move offscreen */
             xwc.width = 1;
         } else {
             xwc.x = tmp_win->highlightxr;
@@ -1043,7 +1013,7 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
 	xwc.width = (tmp_win->name_x - tmp_win->highlightxl);
 	if (Scr->use3Dtitles) xwc.width -= 4;
 	if (xwc.width <= 0) {
-	     xwc.x = Scr->MyDisplayWidth;        /* move offscreen */
+	     xwc.x = Scr->rootw;        /* move offscreen */
 	     xwc.width = 1;
 	} else {
 	     xwc.x = tmp_win->highlightxl;
@@ -1058,7 +1028,7 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
 	if (Scr->TBInfo.nright > 0) xwc.width -= Scr->TitlePadding;
 	if (Scr->use3Dtitles) xwc.width -= 4;
 	if (xwc.width <= 0) {
-	     xwc.x = Scr->MyDisplayWidth;        /* move offscreen */
+	     xwc.x = Scr->rootw;        /* move offscreen */
 	     xwc.width = 1;
 	} else {
 	     xwc.x = tmp_win->highlightxr;
@@ -1109,18 +1079,18 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
  **********************************************************************
  */
 
-void
-fullzoom(tmp_win,flag)
-TwmWindow *tmp_win;
-int flag;
+void fullzoom(TwmWindow *tmp_win, int flag)
 {
     Window      junkRoot;
     unsigned int junkbw, junkDepth;
     int basex, basey;
     int border_x, border_y;
     int frame_bw_times_2;
-    int  zwidth = Scr->MyDisplayWidth;
-    int zheight = Scr->MyDisplayHeight;
+    int  zwidth = Scr->rootw;
+    int zheight = Scr->rooth;
+    int tmpX, tmpY, tmpW, tmpH;
+
+
 
 	XGetGeometry(dpy, (Drawable) tmp_win->frame, &junkRoot,
 	        &dragx, &dragy, (unsigned int *)&dragWidth, (unsigned int *)&dragHeight, &junkbw,
@@ -1143,31 +1113,30 @@ int flag;
             border_x = 0;
             border_y = 0;
 	}
-        if (tmp_win->zoomed == flag)
-        {
-            dragHeight = tmp_win->save_frame_height;
-            dragWidth = tmp_win->save_frame_width;
-            dragx = tmp_win->save_frame_x;
-            dragy = tmp_win->save_frame_y;
-            tmp_win->zoomed = ZOOM_NONE;
-        }
-        else
-        {
-                if (tmp_win->zoomed == ZOOM_NONE)
-                {
-                        tmp_win->save_frame_x = dragx;
-                        tmp_win->save_frame_y = dragy;
-                        tmp_win->save_frame_width = dragWidth;
-                        tmp_win->save_frame_height = dragHeight;
-                        tmp_win->zoomed = flag;
-                 }
-                  else
-                            tmp_win->zoomed = flag;
+	if (tmp_win->zoomed == flag)
+	{
+		dragHeight = tmp_win->save_frame_height;
+		dragWidth = tmp_win->save_frame_width;
+		dragx = tmp_win->save_frame_x;
+		dragy = tmp_win->save_frame_y;
+		tmp_win->zoomed = ZOOM_NONE;
+	}
+	else
+	{
+		if (tmp_win->zoomed == ZOOM_NONE)
+		{
+			tmp_win->save_frame_x = dragx;
+			tmp_win->save_frame_y = dragy;
+			tmp_win->save_frame_width = dragWidth;
+			tmp_win->save_frame_height = dragHeight;
+			tmp_win->zoomed = flag;
+		}
+		else
+			tmp_win->zoomed = flag;
 
+		frame_bw_times_2 = 2*tmp_win->frame_bw;
 
-	frame_bw_times_2 = 2*tmp_win->frame_bw;
-
-        switch (flag)
+		switch (flag)
         {
         case ZOOM_NONE:
             break;
@@ -1216,16 +1185,32 @@ int flag;
         RaiseWindow(tmp_win);
 
     ConstrainSize(tmp_win, &dragWidth, &dragHeight);
-
+#ifdef BETTERZOOM 
+	if (flag == F_ZOOM)
+	{
+		if (dragy + dragHeight < tmp_win->save_frame_y + tmp_win->save_frame_height)
+			dragy = tmp_win->save_frame_y + tmp_win->save_frame_height - dragHeight;
+	}
+#endif
     SetupWindow (tmp_win, dragx , dragy , dragWidth, dragHeight, -1);
 /* I don't understand the reason of this. Claude.
     XUngrabPointer (dpy, CurrentTime);
 */
     XUngrabServer (dpy);
+
+   XQueryPointer(dpy, 
+		 tmp_win->w, 
+		 &junkRoot, &junkRoot, 
+		 &tmpX, &tmpY, &tmpW, &tmpH, &junkDepth);
+   if (tmp_win->frame_x > tmpX ||
+       tmp_win->frame_x + tmp_win->frame_width < tmpX ||
+       tmp_win->frame_y > tmpY ||
+       tmp_win->frame_y + tmp_win->frame_height < tmpY) {
+      XWarpPointer(dpy, Scr->Root, tmp_win->w, 0, 0, 0, 0, 0, 0);
+   }
 }
 
-void savegeometry (tmp_win)
-TwmWindow *tmp_win;
+void savegeometry (TwmWindow *tmp_win)
 {
     if (!tmp_win) return;
     tmp_win->savegeometry.x      = tmp_win->frame_x;
@@ -1234,13 +1219,13 @@ TwmWindow *tmp_win;
     tmp_win->savegeometry.height = tmp_win->frame_height;
 }
 
-void restoregeometry (tmp_win)
-TwmWindow *tmp_win;
+void restoregeometry (TwmWindow *tmp_win)
 {
-    int x, y, w, h;
+    int x, y;
+    unsigned int w, h;
 
     if (!tmp_win) return;
-    if (tmp_win->savegeometry.width == -1) return;
+    if (tmp_win->savegeometry.width == (unsigned int)-1) return;
     x = tmp_win->savegeometry.x;
     y = tmp_win->savegeometry.y;
     w = tmp_win->savegeometry.width;
@@ -1248,8 +1233,7 @@ TwmWindow *tmp_win;
     SetupWindow (tmp_win, x, y, w, h, -1);
 }
 
-SetFrameShape (tmp)
-    TwmWindow *tmp;
+void SetFrameShape (TwmWindow *tmp)
 {
     /*
      * see if the titlebar needs to move
@@ -1327,6 +1311,156 @@ SetFrameShape (tmp)
     }
 }
 
+void ChangeSize (char *in_string, TwmWindow *tmp_win)
+{
+  int i=0, j=0, change=0, size=0;
+  char tmp_string[10], tmp_string2[10];
+  char operator ='\0';
+  char size_string[10];
+  int rx, ry, wx, wy, mr;
+  Window  rr, cr;
+
+  if (isdigit(in_string[0]))
+    {
+      while(in_string[i] != 'x')
+	{
+	  tmp_string[i] = in_string[i];
+	  i++;
+	}
+      tmp_string[i]='\0';
+      i++;
+      while(in_string[i] != '\0')
+	{
+	  tmp_string2[j] = in_string[i];
+	  i++;
+	  j++;
+	}
+
+      wx = atoi(tmp_string);
+      wy = atoi(tmp_string2);
+
+      if (wy < tmp_win->title_height+1)
+	wy = tmp_win->title_height+1;
+
+      SetupWindow (tmp_win, tmp_win->frame_x, tmp_win->frame_y,
+		       wx, wy+tmp_win->title_height, -1);
+    }
+  else
+    {
+      while(in_string[i] != ' ')
+	{
+	  tmp_string[i] = in_string[i];
+	  i++;
+	}
+      tmp_string[i]='\0';
+
+      i++;
+      operator = in_string[i];
+      i++;
+
+      while(in_string[i] != '\0')
+	{
+	  size_string[j] = in_string[i];
+	  i++;
+	  j++;
+	}
+      size_string[j] = '\0';
+
+      change = atoi(size_string);
+
+      if (operator == '-')
+	{
+	  change = 0 - change;
+	}
+      else if (operator != '+')
+	{
+	  /* error */
+	  fprintf (stderr, "%s: Bad argument to f.changesize\n", ProgramName);
+	  return;
+	}
+
+      if (strcmp("bottom", tmp_string) == 0)
+	{
+	  size = tmp_win->frame_height + change;
+
+	  if (size < (tmp_win->title_height+1))
+	    size = tmp_win->title_height+1;
+
+	  SetupWindow (tmp_win, tmp_win->frame_x, tmp_win->frame_y,
+		       tmp_win->frame_width, size , 
+		       -1);
+
+	  XQueryPointer(dpy, tmp_win->w, &rr, &cr, &rx, &ry, &wx, &wy, 
+			(unsigned int*)&mr);
+
+	  if ((wy+tmp_win->title_height) > size)
+	    XWarpPointer(dpy, None, tmp_win->w, 0, 0, 0, 0, 0, 0);
+	}
+      else if (strcmp("top", tmp_string) == 0)
+	{
+	  size = tmp_win->frame_height + change;
+
+	  if (size < (tmp_win->title_height+1))
+	    size = tmp_win->title_height+1;
+
+	  SetupWindow (tmp_win, tmp_win->frame_x, (tmp_win->frame_y - change),
+		       tmp_win->frame_width, size, 
+		       -1);
+      
+	  XQueryPointer(dpy, tmp_win->w, &rr, &cr, &rx, &ry, &wx, &wy, 
+			(unsigned int*)&mr);
+
+	  if ((wy + tmp_win->title_height) > size)
+	    XWarpPointer(dpy, None, tmp_win->w, 0, 0, 0, 0, 0, 0);
+
+
+	}
+      else if (strcmp("left", tmp_string) == 0)
+	{
+	  size = tmp_win->frame_width + change;
+
+	  if (size < 1)
+	    size = 1;
+
+	  SetupWindow (tmp_win, (tmp_win->frame_x - change), tmp_win->frame_y,
+		       size, tmp_win->frame_height, 
+		       -1);
+      
+	  XQueryPointer(dpy, tmp_win->w, &rr, &cr, &rx, &ry, &wx, &wy, 
+			(unsigned int*)&mr);
+
+	  if (wx > size)
+	    XWarpPointer(dpy, None, tmp_win->w, 0, 0, 0, 0, 0, 0);
+
+      
+	}
+      else if (strcmp("right", tmp_string) == 0)
+	{
+	  size = tmp_win->frame_width + change;
+
+	  if (size < 1)
+	    size = 1;
+
+	  SetupWindow (tmp_win, tmp_win->frame_x, tmp_win->frame_y,
+		       size, tmp_win->frame_height, 
+		       -1);
+
+	  XQueryPointer(dpy, tmp_win->w, &rr, &cr, &rx, &ry, &wx, &wy, 
+			(unsigned int*)&mr);
+
+	  if (wx > size)
+	    XWarpPointer(dpy, None, tmp_win->w, 0, 0, 0, 0, 0, 0);
+	  
+	}
+      else
+	{
+	  /* error */
+	  fprintf (stderr, "%s: Bad argument to f.changesize\n", ProgramName);
+	  return;
+	}
+    }
+}
+
 /*
  * Squeezed Title:
  * 
