@@ -66,45 +66,64 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #endif
+
+#ifdef GNOME
+#  include "gnome.h"
+#endif /* GNOME */
 #include "list.h"
 #include "menus.h"
 #include "iconmgr.h"
+#include "vscreen.h"
 #include "workmgr.h"
 
-typedef struct _StdCmap {
+#define ICONIFY_NORMAL  0
+#define ICONIFY_MOSAIC  1
+#define ICONIFY_ZOOMIN  2
+#define ICONIFY_ZOOMOUT 3
+#define ICONIFY_SWEEP   4
+
+struct _StdCmap {
     struct _StdCmap *next;		/* next link in chain */
     Atom atom;				/* property from which this came */
     int nmaps;				/* number of maps below */
     XStandardColormap *maps;		/* the actual maps */
-} StdCmap;
+};
 
 #define SIZE_HINDENT 10
 #define SIZE_VINDENT 2
 
-typedef struct _TitlebarPixmaps {
+struct _TitlebarPixmaps {
     Pixmap xlogo;
     Pixmap resize;
     Pixmap question;
     Pixmap menu;
     Pixmap delete;
-} TitlebarPixmaps;
+};
 
-typedef struct ScreenInfo
+struct ScreenInfo
 {
     int screen;			/* the default screen */
     int d_depth;		/* copy of DefaultDepth(dpy, screen) */
     Visual *d_visual;		/* copy of DefaultVisual(dpy, screen) */
     int Monochrome;		/* is the display monochrome ? */
-    int MyDisplayX;		/* The x coordinate of the root window if captive */
-    int MyDisplayY;		/* The y coordinate of the root window if captive */
-    int MyDisplayWidth;		/* my copy of DisplayWidth(dpy, screen) */
-    int MyDisplayHeight;	/* my copy of DisplayHeight(dpy, screen) */
+    int rootx;		        /* The x coordinate of the root window */
+    int rooty;		        /* The y coordinate of the root window */
+    int rootw;		        /* my copy of DisplayWidth(dpy, screen) */
+    int rooth;	                /* my copy of DisplayHeight(dpy, screen) */
+
+    int crootx;		        /* The x coordinate of the captive root window if any */
+    int crooty;		        /* The y coordinate of the captive root window if any */
+    int crootw;		        /* my copy of DisplayWidth(dpy, screen) */
+    int crooth;	                /* my copy of DisplayHeight(dpy, screen) */
+
     int MaxWindowWidth;		/* largest window to allow */
     int MaxWindowHeight;	/* ditto */
 
     TwmWindow TwmRoot;		/* the head of the twm window list */
 
     Window Root;		/* the root window */
+    Window RealRoot;		/* the actual root window */
+    Window CaptiveRoot;		/* the captive root window */
     Window SizeWindow;		/* the resize dimensions window */
     Window InfoWindow;		/* the information window */
     Window WindowMask;		/* the window masking the screen at startup */
@@ -120,7 +139,7 @@ typedef struct ScreenInfo
     Image *UnknownImage;	/* the unknown icon pixmap */
     Pixmap siconifyPm;		/* the icon manager iconify pixmap */
     Pixmap pullPm;		/* pull right menu icon */
-    int	pullW, pullH;		/* size of pull right menu icon */
+    unsigned int pullW, pullH;	/* size of pull right menu icon */
     char *HighlightPixmapName;	/* name of the hilite image if any */
 
     MenuRoot *MenuList;		/* head of the menu list */
@@ -130,6 +149,14 @@ typedef struct ScreenInfo
     MenuRoot *Workspaces;	/* the TwmWorkspaces menu */
     MenuRoot *AllWindows;	/* the TwmAllWindows menu */
 
+  /*Added by dl 2004 */
+    MenuRoot *AllIcons;         /* the TwmAllIcons menu */
+
+  /******************************************************/
+  /* Added by Dan Lilliehorn (dl@dl.nu) 2000-02-29)     */
+    MenuRoot *Keys;             /* the TwmKeys menu     */
+    MenuRoot *Visible;          /* thw TwmVisible menu  */
+
     TwmWindow *Ring;		/* one of the windows in window ring */
     TwmWindow *RingLeader;	/* current window in ring */
 
@@ -184,7 +211,7 @@ typedef struct ScreenInfo
     short IconRegionJustification;	/* J_LEFT, J_CENTER J_RIGHT or J_BORDER */
     short IconRegionAlignement;	/* J_TOP, J_CENTER, J_BOTTOM or J_BORDER */
     short TitleJustification;	/* J_LEFT, J_CENTER or J_RIGHT */
-    short SmartIconify;
+    short IconifyStyle;         /* ICONIFY_* */
     int   MaxIconTitleWidth;	/* */
 
     Cursor TitleCursor;		/* title bar cursor */
@@ -202,6 +229,11 @@ typedef struct ScreenInfo
 
     WorkSpaceMgr workSpaceMgr;
     short	workSpaceManagerActive;
+
+    virtualScreen *vScreenList;
+    virtualScreen *currentvs;
+    name_list     *VirtualScreens;
+
     name_list	*OccupyAll;	/* list of window names occupying all workspaces at startup */
     name_list	*UnmapByMovingFarAway;
     name_list	*DontSetInactive;
@@ -231,7 +263,7 @@ typedef struct ScreenInfo
     short	XMoveGrid, YMoveGrid;
     short	FastServer;
     short	CenterFeedbackWindow;
-    short	SchrinkIconTitles;
+    short	ShrinkIconTitles;
     short	AutoRaiseIcons;
     short       AutoFocusToTransients; /* kai */
     short       PackNewWindows;
@@ -266,11 +298,13 @@ typedef struct ScreenInfo
     name_list *IconManagerHighlightL;	/* icon manager highlight colors */
     name_list *SqueezeTitleL;		/* windows of which to squeeze title */
     name_list *DontSqueezeTitleL;	/* windows of which not to squeeze */
+    name_list *AlwaysSqueezeToGravityL;	/* windows which should squeeze toward gravity */
     name_list *WindowRingL;	/* windows in ring */
     name_list *WindowRingExcludeL;      /* windows excluded from ring */
     name_list *WarpCursorL;	/* windows to warp cursor to on deiconify */
     name_list *DontSave;
     name_list *WindowGeometries;
+    name_list *IgnoreTransientL;
 
     name_list *OpaqueMoveList;
     name_list *NoOpaqueMoveList;
@@ -372,6 +406,7 @@ typedef struct ScreenInfo
     short NoIconManagers;	/* Don't create any icon managers */
     short ClientBorderWidth;	/* respect client window border width */
     short SqueezeTitle;		/* make title as small as possible */
+    short AlwaysSqueezeToGravity; /* squeeze toward gravity */
     short HaveFonts;		/* set if fonts have been loaded */
     short FirstTime;		/* first time we've read .twmrc */
     short CaseSensitive;	/* be case-sensitive when sorting names */
@@ -386,10 +421,16 @@ typedef struct ScreenInfo
     short IgnoreLockModifier;	/* Should we ignore the lock modifier */
     unsigned int IgnoreModifier;
     short IgnoreCaseInMenuSelection;	/* Should we ignore case in menu selection */
+    short NoWarpToMenuTitle; /* warp cursor to clipped menu title */
+    short NoImagesInWorkSpaceManager;   /* do not display mini images of the desktop background images on WSmap */
 
     FuncKey FuncKeyRoot;
     FuncButton FuncButtonRoot;
-} ScreenInfo;
+
+#ifdef GNOME
+    GnomeData *gnomedata;
+#endif /* GNOME */
+};
 
 extern int MultiScreen;
 extern int NumScreens;
