@@ -32,6 +32,9 @@
 #include <rplay.h>
 #include <string.h>
 #include <stdio.h>
+#include <ctype.h>
+
+#include "sound.h"
 
 char *eventNames[] =
 {
@@ -85,11 +88,29 @@ static int startup_sound = NEVENTS -2;
 static int exit_sound = NEVENTS -1;
 static char hostname[200];
 
+/*
+ * Function to trim away spaces at the start and end of a string
+ */
+static char *
+trim_spaces(char *str)
+{
+    if (str != NULL) {
+	char *p = str + strlen(str);
+	while(*str != '\0' && *str != '\r' && *str != '\n' && isspace(*str))
+	    str++;
+	/* Assume all line end characters are at the end */
+	while(p > str && isspace(p[-1]))
+	    p--;
+	*p = '\0';
+    }
+    return str;
+}
+
 /*
  * initialize
  */
-static
-sound_init ()
+static void
+sound_init (void)
 {
     int i;
     FILE *fl;
@@ -127,20 +148,20 @@ sound_init ()
     if (fl == NULL)
 	return;
     while (fgets (buffer, 100, fl) != NULL) {
-	token = strtok (buffer, ": \t");
-	if (token == NULL)
+	token = trim_spaces(strtok (buffer, ": \t"));
+	if (token == NULL || *token == '#')
 	    continue;
-        for (i = 0; i < NEVENTS; i++) {
+	for (i = 0; i < NEVENTS; i++) {
 	    if (strcmp (token, eventNames[i]) == 0) {
-	        token = strtok (NULL, " \t\r\n");
-	        if (token == NULL || *token == '#' || isspace (*token))
+		token = trim_spaces(strtok (NULL, "\r\n"));
+		if (token == NULL || *token == '#')
 		    continue;
-	        rp[i] = rplay_create (RPLAY_PLAY);
-	        if (rp[i] == NULL) {
+		rp[i] = rplay_create (RPLAY_PLAY);
+		if (rp[i] == NULL) {
 		    rplay_perror ("create");
 		    continue;
 		}
-	        if (rplay_set(rp[i], RPLAY_INSERT, 0, RPLAY_SOUND, token, NULL)
+		if (rplay_set(rp[i], RPLAY_INSERT, 0, RPLAY_SOUND, token, NULL)
 		    < 0)
 		    rplay_perror ("rplay");
 	    }
@@ -153,8 +174,8 @@ sound_init ()
 /*
  * Play sound
  */
-play_sound (snd)
-int snd;
+void
+play_sound (int snd)
 {
     if (snd > NEVENTS) return;
     if (sound_state == 0)
@@ -169,12 +190,14 @@ int snd;
 	rplay_perror ("create");
 }
 
-play_startup_sound()
+void
+play_startup_sound(void)
 {
     play_sound(startup_sound);
 }
 
-play_exit_sound()
+void
+play_exit_sound(void)
 {
     play_sound(exit_sound);
 }
@@ -182,7 +205,8 @@ play_exit_sound()
 /*
  * Toggle the sound on/off
  */
-toggle_sound ()
+void
+toggle_sound (void)
 {
     sound_state ^= 1;
 }
@@ -191,7 +215,8 @@ toggle_sound ()
 /*
  * Re-read the sounds mapping file
  */
-reread_sounds ()
+void
+reread_sounds (void)
 {
     sound_init ();
 }
@@ -199,8 +224,8 @@ reread_sounds ()
 /*
  * Set the SoundHost and force the sound_fd to be re-opened.
  */
-set_sound_host(host)
-char	*host;
+void
+set_sound_host(char *host)
 {
 	strcpy(hostname, host);
 	if (sound_fd != 0)
