@@ -87,8 +87,8 @@
 #    include <X11/Xfuncs.h>
 #endif
 #endif
+#include "types.h"
 #include "list.h"
-#include "util.h"
 
 #ifndef WithdrawnState
 #define WithdrawnState 0
@@ -106,7 +106,7 @@
 #define SIGNAL_RETURN return
 #endif
 
-typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
+typedef SIGNAL_T (*SigProc)(int); /* type of function returned by signal() */
 
 #if defined(USE_SIGNALS) && defined(SVR4) && !defined(__sgi)
 #define signal sigset
@@ -122,7 +122,7 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 
 #define NULLSTR ((char *) NULL)
 
-#define MAX_BUTTONS	5	/* max mouse buttons supported */
+#define MAX_BUTTONS	6	/* max mouse buttons supported */
 
 /* info stings defines */
 #define INFO_LINES 30
@@ -145,7 +145,8 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 #define C_NAME		6
 #define C_IDENTIFY      7
 #define C_ALTERNATE     8
-#define NUM_CONTEXTS	9
+#define C_WORKSPACE	9
+#define NUM_CONTEXTS	10
 
 #define C_WINDOW_BIT	(1 << C_WINDOW)
 #define C_TITLE_BIT	(1 << C_TITLE)
@@ -155,9 +156,11 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 #define C_ICONMGR_BIT	(1 << C_ICONMGR)
 #define C_NAME_BIT	(1 << C_NAME)
 #define C_ALTER_BIT	(1 << C_ALTERNATE)
+#define C_WORKSPACE_BIT	(1 << C_WORKSPACE)
 
 #define C_ALL_BITS	(C_WINDOW_BIT | C_TITLE_BIT | C_ICON_BIT |\
-			 C_ROOT_BIT | C_FRAME_BIT | C_ICONMGR_BIT)
+			 C_ROOT_BIT | C_FRAME_BIT | C_ICONMGR_BIT |\
+			 C_WORKSPACE_BIT)
 
 /* modifiers for button presses */
 #define MOD_SIZE	((ShiftMask | ControlMask | Mod1Mask \
@@ -186,7 +189,7 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 #define MinSize(a, b)  (((a) > (b)) ? (b) : (a))
 #endif
 
-typedef struct MyFont
+struct MyFont
 {
 #ifdef I18N    
     char *basename;			/* name of the font */
@@ -199,22 +202,22 @@ typedef struct MyFont
 #endif
     int height;			/* height of the font */
     int y;			/* Y coordinate to draw characters */
-} MyFont;
+};
 
-typedef struct ColorPair
+struct ColorPair
 {
     Pixel fore, back, shadc, shadd;
-} ColorPair;
+};
 
 typedef enum {on, off} ButtonState;
 
-typedef struct _TitleButtonFunc {
+struct _TitleButtonFunc {
     int func;                         /* function to execute */
     char *action;                     /* optional action arg */
     struct MenuRoot *menuroot;                /* menu to pop on F_MENU */
-} TitleButtonFunc;
+};
 
-typedef struct _TitleButton {
+struct _TitleButton {
     struct _TitleButton *next;		/* next link in chain */
     char *name;				/* bitmap name in case of deferal */
     Image *image;			/* image to display in button */
@@ -223,19 +226,19 @@ typedef struct _TitleButton {
     int dstx, dsty;			/* to where to start copying */
     Bool rightside;			/* t: on right, f: on left */
     TitleButtonFunc funs[MAX_BUTTONS];  /* funcs assoc'd to each button */
-} TitleButton;
+};
 
-typedef struct _TBWindow {
+struct _TBWindow {
     Window window;			/* which window in this frame */
     Image *image;			/* image to display in button */
     TitleButton *info;			/* description of this window */
-} TBWindow;
+};
 
-typedef struct _SqueezeInfo {
+struct _SqueezeInfo {
     int justify;			/* left, center, right */
     int num;				/* signed pixel count or numerator */
     int denom;				/* 0 for pix count or denominator */
-} SqueezeInfo;
+};
 
 #define J_UNDEF			0
 #define J_LEFT			1
@@ -248,65 +251,65 @@ typedef struct _SqueezeInfo {
 /* Colormap window entry for each window in WM_COLORMAP_WINDOWS
  * ICCCM property.
  */
-typedef struct TwmColormap
-{	
+struct TwmColormap
+{
     Colormap c;			/* Colormap id */
     int state;			/* install(ability) state */
     unsigned long install_req;	/* request number which installed it */
     Window w;			/* window causing load of color table */
     int refcnt;
-} TwmColormap;
+};
 
 #define CM_INSTALLABLE		1
 #define CM_INSTALLED		2
 #define CM_INSTALL		4
 
-typedef struct ColormapWindow
+struct ColormapWindow
 {
     Window w;			/* Window id */
     TwmColormap *colormap;	/* Colormap for this window */
     int visibility;		/* Visibility of this window */
     int refcnt;
-} ColormapWindow;
+};
 
-typedef struct Colormaps
+struct Colormaps
 {
     ColormapWindow **cwins;	/* current list of colormap windows */
     int number_cwins;		/* number of elements in current list */
     char *scoreboard;		/* conflicts between installable colortables */
-} Colormaps;
+};
 
 #define ColormapsScoreboardLength(cm) ((cm)->number_cwins * \
 				       ((cm)->number_cwins - 1) / 2)
 
-typedef struct WindowRegion {
+struct WindowRegion {
     struct WindowRegion	*next;
     int			x, y, w, h;
     int			grav1, grav2;
     name_list           *clientlist;
     struct WindowEntry	*entries;
-} WindowRegion;
+};
 
-typedef struct WindowEntry {
+struct WindowEntry {
     struct WindowEntry	*next;
     int			x, y, w, h;
     struct TwmWindow	*twm_win;
     short 		used;
-} WindowEntry;
+};
 
-typedef struct _WindowBox {
+struct _WindowBox {
     struct _WindowBox	*next;
     char		*name;
     char		*geometry;
     name_list           *winlist;
     Window		window;
     struct TwmWindow	*twmwin;
-} WindowBox;
+};
 
 /* for each window that is on the display, one of these structures
  * is allocated and linked into a list 
  */
-typedef struct TwmWindow
+struct TwmWindow
 {
     struct TwmWindow *next;	/* next twm window */
     struct TwmWindow *prev;	/* previous twm window */
@@ -316,7 +319,7 @@ typedef struct TwmWindow
     Window title_w;		/* the title bar window */
     Window hilite_wl;		/* the left hilite window */
     Window hilite_wr;		/* the right hilite window */
-    Window lolite_wl;		/* the right lolite window */
+    Window lolite_wl;		/* the left lolite window */
     Window lolite_wr;		/* the right lolite window */
     Cursor curcurs;		/* current resize cursor */
     Pixmap gray;
@@ -324,23 +327,23 @@ typedef struct TwmWindow
     name_list *iconslist;	/* the current list of icons */
     int frame_x;		/* x position of frame */
     int frame_y;		/* y position of frame */
-    int frame_width;		/* width of frame */
-    int frame_height;		/* height of frame */
+    unsigned int frame_width;	/* width of frame */
+    unsigned int frame_height;	/* height of frame */
     int frame_bw;		/* borderwidth of frame */
     int frame_bw3D;		/* 3D borderwidth of frame */
     int actual_frame_x;		/* save frame_y of frame when squeezed */
     int actual_frame_y;		/* save frame_x of frame when squeezed */
-    int actual_frame_width;	/* save width of frame when squeezed */
-    int actual_frame_height;	/* save height of frame when squeezed */
+    unsigned int actual_frame_width;  /* save width of frame when squeezed */
+    unsigned int actual_frame_height; /* save height of frame when squeezed */
     int title_x;
     int title_y;
-    int title_height;		/* height of the title bar */
-    int title_width;		/* width of the title bar */
+    unsigned int title_height;	/* height of the title bar */
+    unsigned int title_width;	/* width of the title bar */
     char *full_name;		/* full name of the window */
     char *name;			/* name of the window */
     char *icon_name;            /* name of the icon */
     int name_x;			/* start x of name text */
-    int name_width;		/* width of name text */
+    unsigned int name_width;	/* width of name text */
     int highlightxl;		/* start of left highlight window */
     int highlightxr;		/* start of right highlight window */
     int rightx;			/* start of right buttons */
@@ -371,14 +374,15 @@ typedef struct TwmWindow
     short ontoppriority;	/* how much on top should that be */
     short iconify_by_unmapping;	/* unmap window to iconify it */
     short iconmgr;		/* this is an icon manager window */
+    short wspmgr;		/* this is a workspace manager manager window */
     short transient;		/* this is a transient window */
     Window transientfor;	/* window contained in XA_XM_TRANSIENT_FOR */
     short titlehighlight;	/* should I highlight the title bar */
     struct IconMgr *iconmgrp;	/* pointer to it if this is an icon manager */
     int save_frame_x;		/* x position of frame */
     int save_frame_y;		/* y position of frame */
-    int save_frame_width;	/* width of frame */
-    int save_frame_height;	/* height of frame */
+    unsigned int save_frame_width;  /* width of frame */
+    unsigned int save_frame_height; /* height of frame */
     short zoomed;		/* is the window zoomed? */
     short wShaped;		/* this window has a bounding shape */
     unsigned long protocols;	/* which protocols this window handles */
@@ -396,6 +400,7 @@ typedef struct TwmWindow
     short UnmapByMovingFarAway;
     short AutoSqueeze;
     short StartSqueezed;
+    short AlwaysSqueezeToGravity;
     short DontSetInactive;
     Bool hasfocusvisible;	/* The window has visivle focus*/
     int  occupation;
@@ -406,8 +411,11 @@ typedef struct TwmWindow
     Bool iswinbox;
     struct {
 	int x, y;
-	int width, height;
+	unsigned int width, height;
     } savegeometry;
+    struct virtualScreen *vs;
+    struct virtualScreen *oldvs;
+    struct virtualScreen *savevs;
 
 #ifdef X11R6
     Bool nameChanged;	/* did WM_NAME ever change? */
@@ -416,10 +424,10 @@ typedef struct TwmWindow
     Bool heightEverChangedByUser;
 #endif
 
-} TwmWindow;
+};
 
 #ifdef X11R6
-typedef struct TWMWinConfigEntry
+struct TWMWinConfigEntry
 {
     struct TWMWinConfigEntry *next;
     int tag;
@@ -443,7 +451,7 @@ typedef struct TWMWinConfigEntry
     int occupation;
    /* ====================================================================== */
 
-} TWMWinConfigEntry;
+};
 #endif
 
 #define DoesWmTakeFocus		(1L << 0)
@@ -487,10 +495,14 @@ extern char *malloc(), *calloc(), *realloc(), *getenv();
 extern void free();
 #endif
 #endif
-extern void Reborder();
-extern SIGNAL_T Done();
-void ComputeCommonTitleOffsets();
-void ComputeWindowTitleOffsets(), ComputeTitleLocation();
+extern void Reborder(Time time);
+extern SIGNAL_T Done(int signum);
+void ComputeCommonTitleOffsets(void);
+void ComputeWindowTitleOffsets(TwmWindow *tmp_win, unsigned int width,
+			       Bool squeeze);
+void ComputeTitleLocation(register TwmWindow *tmp);
+void CreateFonts(void);
+void RestoreWithdrawnLocation (TwmWindow *tmp);
 extern char *ProgramName;
 extern Display *dpy;
 #ifdef X11R6
@@ -513,6 +525,7 @@ extern XContext MenuContext;
 extern XContext IconManagerContext;
 extern XContext ScreenContext;
 extern XContext ColormapContext;
+extern XContext VirtScreenContext;
 
 extern char *Home;
 extern int HomeLen;
@@ -526,16 +539,14 @@ extern int JunkX;
 extern int JunkY;
 extern unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
 extern XGCValues Gcv;
-extern int InfoLines,InfoWidth,InfoHeight;
+extern int InfoLines;
+extern unsigned int InfoWidth,InfoHeight;
 extern char Info[][INFO_SIZE];
 extern int Argc;
 extern char **Argv;
 #ifndef VMS
 extern char **Environ;
 #endif
-extern void NewFontCursor();
-extern Pixmap CreateMenuIcon();
-extern Pixmap Create3DMenuIcon();
 
 extern Bool ErrorOccurred;
 extern XErrorEvent LastErrorEvent;
@@ -543,7 +554,10 @@ extern XErrorEvent LastErrorEvent;
 #define ResetError() (ErrorOccurred = False)
 
 extern Bool RestartPreviousState;
-extern Bool GetWMState();
+extern Bool GetWMState(Window w, int *statep, Window *iwp);
+
+extern Bool RestartFlag;	/* Flag that is set when SIGHUP is caught */
+extern void DoRestart(Time t);	/* Function to perform a restart */
 
 extern Atom _XA_MIT_PRIORITY_COLORS;
 extern Atom _XA_WM_CHANGE_STATE;
